#include "ball.h"

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

/******************************************************************
 *                           Constructor                          *
 ******************************************************************/
ball::ball()
{
   posX = FIELD_X / 2.0;
   posY = 0;
   posZ = FIELD_Z / 2.0;
   
   prvX = posX;
   prvY = posY;
   prvZ = posZ;
   
   angleX = 0;
   angleY = 0;
   angleZ = 0;
   
   dirX = 0;
   dirZ = 0;
   
   velocity = 0;
   velY = 0;

   /* Set its type */
   type = FOBJECT_TYPE_BALL;
   
   /* Load Ball Model */
   model = new aniModel();
   model->loadModel("models/ball/model.cfg");
   /* Since there aren't animations, the bounding box is static */
   model->calculateBoundingBox(min, max);
}

/******************************************************************
 *                            Destructor                          *
 ******************************************************************/
ball::~ball()
{
   delete(model);
}

/******************************************************************
 *                             getPosX                            *
 ******************************************************************/
float ball::getPosX()
{
   return(posX);
}

/******************************************************************
 *                             getPosY                            *
 ******************************************************************/
float ball::getPosY()
{
   return(posY);
}

/******************************************************************
 *                             getPosZ                            *
 ******************************************************************/
float ball::getPosZ()
{
   return(posZ);
}

/******************************************************************
 *                      getPreviousPosition                       *
 ******************************************************************/
void ball::getPreviousPosition(float& x, float& y, float& z)
{
   x = prvX;
   y = prvY;
   z = prvZ;
}

/******************************************************************
 *                      setPreviousPosition                       *
 ******************************************************************/
void ball::setPreviousPosition(float x, float y, float z)
{
   prvX = x;
   prvY = y;
   prvZ = z;
}

/******************************************************************
 *                           setPosition                          *
 ******************************************************************/
void ball::setPosition(float x, float y, float z)
{
   posX = x;
   posY = y;
   posZ = z;
}

/******************************************************************
 *                             setPosX                            *
 ******************************************************************/
void ball::setPosX(float x)
{
   posX = x;
}

/******************************************************************
 *                             setPosY                            *
 ******************************************************************/
void ball::setPosY(float y)
{
   posY = y;
}

/******************************************************************
 *                             setPosZ                            *
 ******************************************************************/
void ball::setPosZ(float z)
{
   posZ = z;
}

/******************************************************************
 *                            getAngleX                           *
 ******************************************************************/
float ball::getAngleX()
{
   return(angleX);
}

/******************************************************************
 *                            getAngleY                           *
 ******************************************************************/
float ball::getAngleY()
{
   return(angleY);
}

/******************************************************************
 *                            getAngleZ                           *
 ******************************************************************/
float ball::getAngleZ()
{
   return(angleZ);
}


/******************************************************************
 *                           setAngles                            *
 ******************************************************************/
void ball::setAngles(float x, float y, float z)
{
   angleX = x;
   angleY = y;
   angleZ = z;
}

/******************************************************************
 *                           getVelocity                          *
 ******************************************************************/
float ball::getVelocity()
{
   return(velocity);
}

/******************************************************************
 *                           setVelocity                          *
 ******************************************************************/
void ball::setVelocity(float x, float z, float vel)
{
   dirX = x;
   dirZ = z;
   velocity = vel;
}

/******************************************************************
 *                           getVelocityY                         *
 ******************************************************************/
float ball::getVelocityY()
{
   return(velY);
}

/******************************************************************
 *                           setVelocityY                         *
 ******************************************************************/
void ball::setVelocityY(float y)
{
   velY = y;
}

/******************************************************************
 *                           getOrientX                           *
 ******************************************************************/
float ball::getOrientX()
{
   return(dirX);
}

/******************************************************************
 *                           getOrientZ                           *
 ******************************************************************/
float ball::getOrientZ()
{
   return(dirZ);
}

/******************************************************************
 *                             goBack                             *
 ******************************************************************/
void ball::goBack()
{
   posX = prvX;
   posY = prvY;
   posZ = prvZ;
}

/******************************************************************
 *                             render                             *
 ******************************************************************/
void ball::render()
{
   glPushMatrix();
      glTranslatef(posX, posY+2, posZ);
      glRotatef(angleZ, 0,0,1);
      glRotatef(angleY, 0,1,0);
      glRotatef(angleX, 1,0,0);
      model->render();
   glPopMatrix();
}

/******************************************************************
 *                             render                             *
 ******************************************************************/
void ball::render(float pos[3], float angle[3])
{
   glPushMatrix();
      glTranslatef(pos[0], pos[1]+2, pos[2]);
      glRotatef(angle[2], 0,0,1);
      glRotatef(angle[1], 0,1,0);
      glRotatef(angle[0], 1,0,0);
      model->render();
   glPopMatrix();

   /*model->renderBoundingBox(min, max, pos[0], pos[1]+2, pos[2], 0.0);*/
}

/******************************************************************
 *                          updatePosition                        *
 ******************************************************************/
bool ball::updatePosition()
{
   /* Save previous position */
   prvX = posX;
   prvY = posY;
   prvZ = posZ;
   
   /* Apply velocity */
   posX += velocity*dirX;
   posY += velY;
   posZ += velocity*dirZ;
   
   /* Apply gravity */
   velY -= 0.2;
   
   /* Check ground contact */
   if (posY <= 0)
   {
      /* we dont want the ball to go to japan */
      posY = 0;
      
      /* lose energy to collision in y axis */
      velY = (-0.3) * velY;
      if (fabs(velY) < 0.5)
      {
         velY = 0;
      }
      
      /* lose energy to collision in linear axis */
      velocity = 0.9 * velocity;
      if (velocity <= 0.1) 
      {
         velocity = 0;
      }
   }

   /* Do the Roll */
   if(posY == 0)
   {
      /* Roll Faster */
      angleX += -dirZ*velocity*50;
      angleZ += -dirX*velocity*50;
   }
   else
   {
      /* Roll Slower */
      angleX += -dirZ*velocity*10;
      angleZ += -dirX*velocity*10;
   }

   /* Limit at [0,360) */
   while(fabs(angleX) > 360)
   {
      angleX /= 360.0;
   }
   while(fabs(angleZ) > 360)
   {
      angleZ /= 360.0;
   }


   /* Check field size collision */
   //checkFieldCollision();
   
   return(velocity != 0);
}

/******************************************************************
 *                       checkFieldCollision                      *
 ******************************************************************/
void ball::checkFieldCollision()
{
   if (posX > FIELD_X)
   {  
      posX = FIELD_X;
      dirX = -dirX;
   }
   else if (posX < 0)
   {  
      posX = 0;
      dirX = -dirX;
   }
   
   if (posZ > FIELD_Z)
   {  
      posZ = FIELD_Z;
      dirZ = -dirZ;
   }
   else if (posZ < 0)
   {  
      posZ = 0;
      dirZ = -dirZ;
   }
}
