#ifndef _ball_h
#define _ball_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../etc/animodel.h"
#include "field.h"
#include "fobject.h"

/*! The ball is the game ball implementation with
 * positions and orientations. */
class ball: public fobject
{
   public:
      /* Constructor */
      ball();

      /*! Destructor */
      ~ball();

      /*! get current X position
       * \return -> current X position */
      float getPosX();

      /*! get current Y position
       * \return -> current Y position */
      float getPosY();
   
      /*! get current Z position
       * \return -> current Z position */
      float getPosZ();

      /*! Get previous ball position 
       * \param x -> previous X position
       * \param y -> previous Y position
       * \param z -> previous Z position */
      void getPreviousPosition(float& x, float& y, float& z);

      /*! Set previous ball position 
       * \note -> this function is only used at collision dt
       * \param x -> previous X position
       * \param y -> previous Y position
       * \param z -> previous Z position */
      void setPreviousPosition(float x, float y, float z);

      /*! Set the ball new position
       * \param x -> new x position
       * \param y -> new y position
       * \param z -> new z position */
      void setPosition(float x, float y, float z);
      
      /*! Set the ball new X position
       * \param x -> new X position */
      void setPosX(float x);
      
      /*! Set the ball new Y position
       * \param y -> new Y position */
      void setPosY(float y);
      
      /*! Set the ball new Z position
       * \param z -> new Z position */
      void setPosZ(float z);

      /*! Get the angle of X rotation
       * \return -> current X rotation angle */
      float getAngleX();

      /*! Get the angle of Y rotation
       * \return -> current Y rotation angle */
      float getAngleY();

      /*! Get the angle of Z rotation
       * \return -> current Z rotation angle */
      float getAngleZ();
      
      /*! Set the ball new rotation angles
       * \param x -> new x rotation angle
       * \param y -> new y rotation angle
       * \param z -> new z rotation angle */
      void setAngles(float x, float y, float z);

      /*! Get the Y velocity magnitude of the ball 
       * \return -> current Y velocity of the ball */
      float getVelocityY();

      /*! Get the velocity magnitude of the ball 
       * \return -> current velocity of the ball */
      float getVelocity();
      
      /*! Get the ball X orientation
       * \return x component of orientation vector */
      float getOrientX();

      /*! Get the ball Z orientation
       * \return z component of orientation vector */
      float getOrientZ();

      /*! Set the ball velocity
       * \param x -> X velocity component
       * \param z -> Z velocity component 
       * \param vel -> velocity value */
      void setVelocity(float x, float z, float vel);
      
      /*! Set the ball velocity in Y axis
       * \param y -> Y new velocity  */
      void setVelocityY(float y);
      
      /*! Set the teamPlayer position to the previous position */
      void goBack();
      
      bool updatePosition();

      /*! Render Ball */
      void render();
      
      /*! Render ball with defined position and angle
       * \param pos -> position to render
       * \param angle -> angle to render */
      void render(float pos[3], float angle[3]);

      float min[3];        /**< Ball Min Bounding Box part */
      float max[3];        /**< Ball Max Bounding Box part */

   protected:
      float velocity;      /**< Ball velocity */
      float velY;          /**< Ball velocity in Y axis */
   
      float dirX,          /**< Ball direction on X */
            dirZ;          /**< Ball direction on Z */
   
      float angleX,        /**< Rotation angle on X */
            angleY,        /**< Rotation angle on Y */
            angleZ;        /**< Rotation angle on Z */
      
      float prvX,          /**< Ball previous X position */
            prvY,          /**< Ball previous Y position */
            prvZ;          /**< Ball previous Z position */
         
      float posX,          /**< Ball X position */
            posY,          /**< Ball Y position */
            posZ;          /**< Ball Z Position */

      aniModel* model;     /**< Current Ball Model */

      /*! Checks if teamplayer is outside of the field. If so, it's moved back 
       *  to the field limits and its direction is changed */
      void checkFieldCollision();
};



#endif

