#include "camera.h"
#include "field.h"
#include "../etc/options.h"

#define CAMERA_ATENUATION    0.1  /**< The atenuation value of aceleration */

#define CAMERA_UNDEFINED  -10000  /**< The undefined value */

/******************************************************************
 *                         Constructor                            *
 ******************************************************************/
camera::camera()
{
   theta=45;
   phi=270;
   d=320;
   centerX = FIELD_X / 2.0;
   centerZ = FIELD_Z / 2.0;
   centerY = 30;
   deltaY = 0;
   cameraX = centerX + (float) d * cos(deg2Rad(theta)) * sin(deg2Rad(phi));
   cameraY = centerY + (float) d * sin(deg2Rad(theta));
   cameraZ = centerZ + (float) d * cos(deg2Rad(theta)) * cos(deg2Rad(phi));
   middleMouse = false;
   type = CAMERA_TYPE_NORMAL;
   zoomAc = 0.0;
   phiAc = 0.0;
   thetaAc = 0.0;
   tgtPhi = CAMERA_UNDEFINED;
   minZoom = ZOOM_MIN;
   moveWithMouse = false;
   mousePositionX = -1;
   mousePositionY = -1;
   push();
}

/******************************************************************
 *                          Destructor                            *
 ******************************************************************/
camera::~camera()
{
}

/******************************************************************
 *                             push                               *
 ******************************************************************/
void camera::push()
{
   queue[0] = theta;
   queue[1] = phi;
   queue[2] = d;
   queue[3] = cameraX;
   queue[4] = cameraY;
   queue[5] = cameraZ;
   queue[6] = deltaY;
   queue[7] = centerX;
   queue[8] = centerY;
   queue[9] = centerZ;
   queue[10] = zoomAc;
   queue[11] = thetaAc;
   queue[12] = minZoom;
}

/******************************************************************
 *                              pop                               *
 ******************************************************************/
void camera::pop()
{
   theta = queue[0];
   tgtPhi = queue[1];
   d = queue[2];
   cameraX = queue[3];
   cameraY = queue[4];
   cameraZ = queue[5];
   deltaY = queue[6];
   centerX = queue[7];
   centerY = queue[8];
   centerZ = queue[9];
   zoomAc = queue[10];
   thetaAc = queue[11];
   minZoom = queue[12];
   phiAc = (tgtPhi - phi) / 32.0f;
}

/******************************************************************
 *                             doIO                               *
 ******************************************************************/
bool camera::doIO(Uint8 *keys, Uint8 mBotao, int x, int y, GLfloat varCamera)
{
   options opt;

   /* Keys to Camera Movements */
   if(keys[opt.getKey(XUT_KEY_CAMERA_ZOOM_UP)])  // Increases Zoom
   {
      zoomAc = -(2*varCamera);
   }
   if(keys[opt.getKey(XUT_KEY_CAMERA_ZOOM_DOWN)]) // Decreases Zoom
   {
      zoomAc = (2*varCamera);
   }
   if(keys[opt.getKey(XUT_KEY_CAMERA_ROTATE_RIGHT)]) // Rotate CounterClockWise
   {
      phiAc = -varCamera;
   }
   if(keys[opt.getKey(XUT_KEY_CAMERA_ROTATE_LEFT)]) // Rotate Camera ClockWise
   {
      phiAc = varCamera;
   }
   if((keys[SDLK_PAGEUP]) || 
      (keys[opt.getKey(XUT_KEY_CAMERA_ROTATE_UP)])) // Up Camera
   {
      thetaAc = varCamera;
   }
   if((keys[SDLK_PAGEDOWN]) || 
      (keys[opt.getKey(XUT_KEY_CAMERA_ROTATE_DOWN)])) // Down Camera
   {
      thetaAc = -varCamera;
   }
   if((keys[SDLK_HOME]) || (keys[SDLK_KP7])) // Maximize zoom
   {
      zoomAc = -20;
   }
   if((keys[SDLK_END]) || (keys[SDLK_KP1])) // Minimize zoom
   {
      zoomAc = 20;
   }   
   if((keys[SDLK_INSERT]) || (keys[SDLK_KP0])) //Up view Max
   {
      thetaAc = 10;
   }
   if(keys[SDLK_DELETE]) //Down view Max
   {
      thetaAc = -10;
   }

   if(keys[opt.getKey(XUT_KEY_CAMERA_UP)])
   {
      updateCamera(centerX - 4.0 * sin(deg2Rad(phi)),
                   centerY - 30,
                   centerZ - 4.0 * cos(deg2Rad(phi)), 
                   0.0);
   }
   if(keys[opt.getKey(XUT_KEY_CAMERA_DOWN)])
   {
      updateCamera(centerX + 4.0 * sin(deg2Rad(phi)),
                   centerY-30,
                   centerZ + 4.0 * cos(deg2Rad(phi)), 
                   0.0);
   }
   if(keys[opt.getKey(XUT_KEY_CAMERA_LEFT)])
   {
      updateCamera(centerX - 4.0 * sin(deg2Rad(phi)+deg2Rad(90)),
                   centerY-30,
                   centerZ - 4.0 * cos(deg2Rad(phi)+deg2Rad(90)),
                   0.0);
   }
   if(keys[opt.getKey(XUT_KEY_CAMERA_RIGHT)])
   {
      updateCamera(centerX + 4.0 * sin(deg2Rad(phi)+deg2Rad(90)),
                   centerY-30,
                   centerZ + 4.0 * cos(deg2Rad(phi)+deg2Rad(90)),
                   0.0);
   }

   /* Mouse to move Camera */

   /* Left Edge */
   if(x == 0)  // Turn Clockwise
   {
      phiAc = varCamera; 
   }
   /* Right Edge */
   if(x == opt.getWindowX()-1) // Turn CounterClockWise
   {
      phiAc = -varCamera; 
   }

   /* Left Mouse Button for Camera Position */
   if(moveWithMouse)
   {
      if(mBotao & SDL_BUTTON(1))
      {
         if( (mousePositionX == -1) || (mousePositionY == -1) )
         {
            /* Set the new "initial" position */
            mousePositionX = x;
            mousePositionY = y;
            fixedX = (int)centerX;
            fixedZ = (int)centerZ;
         }
         else
         {
            /* Do the move, based on difference */
            float difX = (mousePositionX - x), 
                  difY = (mousePositionY - y);
            updateCamera(fixedX + 
                         difX * sin(deg2Rad(phi)+deg2Rad(90)) +
                         difY * sin(deg2Rad(phi)),
                         centerY-30,
                         fixedZ +
                         difX * cos(deg2Rad(phi)+deg2Rad(90)) +
                         difY * cos(deg2Rad(phi)),
                         0.0);
         }
      }
      else
      {
         mousePositionX = -1;
         mousePositionY = -1;
      }
   }

   /* Middle Mouse Button Rotation Control */
   if(mBotao & SDL_BUTTON(2))
   {
      if(middleMouse)
      {
         theta += (y - prevY) / 10.0;
         phi += (prevX - x) / 10.0;  // X axis is inverted :~p
      }
      else
      {
         middleMouse = true;  
      }
      prevX = x;
      prevY = y;
   }
   else
   {
      middleMouse = false;
   }

   /* Apply the Zoom accelerations */
   if(zoomAc < 0)
   {
      d += zoomAc;
      zoomAc += CAMERA_ATENUATION;
      if(zoomAc > 0)
      {
         zoomAc = 0;
      }
   }
   else if(zoomAc > 0)
   {
      d += zoomAc;
      zoomAc -= CAMERA_ATENUATION;
      if(zoomAc < 0)
      {
         zoomAc = 0;
      }
   }

   /* Apply the Phi accelerations */
   if(tgtPhi != CAMERA_UNDEFINED)
   {
      phi += phiAc;
      if( (phi == tgtPhi) ||
          ( (phiAc > 0) && (phi > tgtPhi) ) ||
          ( (phiAc < 0) && (phi < tgtPhi) ) )
      {
         /* Got at the target, must stop */
         phi = tgtPhi;
         tgtPhi = CAMERA_UNDEFINED;
         phiAc = 0;
      }
   }
   else if(phiAc < 0)
   {
      phi += phiAc;
      phiAc += CAMERA_ATENUATION;
      if(phiAc > 0)
      {
         phiAc = 0;
      }
   }
   else if(phiAc > 0)
   {
      phi += phiAc;
      phiAc -= CAMERA_ATENUATION;
      if(phiAc < 0)
      {
         phiAc = 0;
      }
   }

   /* Apply the theta accelerations */
   if(thetaAc < 0)
   {
      theta += thetaAc;
      thetaAc += CAMERA_ATENUATION;
      if(thetaAc > 0)
      {
         thetaAc = 0;
      }
   }
   else if(thetaAc > 0)
   {
      theta += thetaAc;
      thetaAc -= CAMERA_ATENUATION;
      if(thetaAc < 0)
      {
         thetaAc = 0;
      }
   }


   /* Verify Limits */

   /* Verify Theta Limits */
   if(theta > 89) 
   {
      theta = 89;
   }
   else if(theta < 0)
   {
      theta = 0;
   }

   /* Verify D limits */
   if(d > minZoom)
   {
      d = minZoom;
   }
   else if(d < ZOOM_MAX)
   {
      d = ZOOM_MAX;
   }

   /* Put Phi on [0-360) range */
   if(phi < 0)
   {
      phi += 360;
   }
   else if(phi >= 360)
   {
      phi = phi - 360;
   }

   return(true);
}

/******************************************************************
 *                           lookAt                               *
 ******************************************************************/
void camera::lookAt()
{
   cameraX = centerX + (float) d * cos(deg2Rad(theta)) * sin(deg2Rad(phi));
   cameraY = centerY + deltaY + (float) d * sin(deg2Rad(theta));
   cameraZ = centerZ + (float) d * cos(deg2Rad(theta)) * cos(deg2Rad(phi));

   gluLookAt(cameraX,cameraY,cameraZ, centerX, centerY, centerZ, 0,1,0);
}

/******************************************************************
 *                          getCameraX                            *
 ******************************************************************/
GLfloat camera::getCameraX()
{
   return(cameraX);
}

/******************************************************************
 *                          getCameraY                            *
 ******************************************************************/
GLfloat camera::getCameraY()
{
   return(cameraY);
}

/******************************************************************
 *                          getCameraZ                            *
 ******************************************************************/
GLfloat camera::getCameraZ()
{
   return(cameraZ);
}

/******************************************************************
 *                           sumTheta                             *
 ******************************************************************/
void camera::sumTheta(GLfloat f)
{
   theta += f;
   if(theta > 89) 
   {
      theta = 89;
   }
   else if(theta < 0)
   {
      theta = 0;
   }
}

/******************************************************************
 *                            setPhi                              *
 ******************************************************************/
void camera::setPhi(GLfloat f)
{
   /* Set the target Phi */
   if(f != tgtPhi)
   {
      tgtPhi = f;
      phiAc = (tgtPhi - phi) / 32.0f;
   }
}

/******************************************************************
 *                            sumPhi                              *
 ******************************************************************/
void camera::sumPhi(GLfloat f)
{
   phi += f;
   if(phi < 0)
   {
      phi += 360;
   }
   else if(phi >= 360)
   {
      phi = phi - 360;
   }
}

/******************************************************************
 *                             sumD                               *
 ******************************************************************/
void camera::sumD(GLfloat f)
{
   d += f;
   if (d < ZOOM_MAX) 
   {
      d = ZOOM_MAX;
   }
   else if (d > ZOOM_MIN)
   {
      d = ZOOM_MIN;
   }
}

/******************************************************************
 *                             getPhi                             *
 ******************************************************************/
GLfloat camera::getPhi()
{
   return(phi);
}

/******************************************************************
 *                            getTheta                            *
 ******************************************************************/
GLfloat camera::getTheta()
{
   return(theta);
}


/******************************************************************
 *                              getD                              *
 ******************************************************************/
GLfloat camera::getD()
{
   return(d);
}


/******************************************************************
 *                           getDeltaY                            *
 ******************************************************************/
GLfloat camera::getDeltaY()
{
   return(deltaY);
}


/******************************************************************
 *                           getCenterX                           *
 ******************************************************************/
GLfloat camera::getCenterX()
{
   return(centerX);
}

/******************************************************************
 *                           getCenterY                           *
 ******************************************************************/
GLfloat camera::getCenterY()
{
   return(centerY);
}

/******************************************************************
 *                           getCenterZ                           *
 ******************************************************************/
GLfloat camera::getCenterZ()
{
   return(centerZ);
}

/******************************************************************
 *                          defineCameraType                      *
 ******************************************************************/
void camera::defineCameraType(int value)
{
   type = value;
}

/******************************************************************
 *                           defineMinZoom                        *
 ******************************************************************/
void camera::defineMinZoom(GLfloat value)
{
   minZoom = value;
}

/******************************************************************
 *                        setMoveWithMouse                        *
 ******************************************************************/
void camera::setMoveWithMouse(bool value)
{
   moveWithMouse = value;
   if(!moveWithMouse)
   {
      mousePositionX = -1;
      mousePositionY = -1;
   }
}

/******************************************************************
 *                        getMoveWithMouse                        *
 ******************************************************************/
bool camera::getMoveWithMouse()
{
   return(moveWithMouse);
}

/******************************************************************
 *                          updateCamera                          *
 ******************************************************************/
void camera::updateCamera(GLfloat characterX, GLfloat characterY, 
                          GLfloat characterZ, GLfloat characterOrientation)
{
   centerX = characterX;
   centerZ = characterZ;
   centerY = characterY + 30;

   if( (type == CAMERA_TYPE_DRIVE) && (characterOrientation != phi) )
   {
      /* Make the camera follow character rotations */
      GLfloat variation = fabs(characterOrientation - phi);
      GLfloat othVariation = fabs(360-variation);
      if( (variation <= DELTA_CAMERA) || (othVariation <= DELTA_CAMERA) )
      {
         phi = characterOrientation;
      }
      else if(phi > characterOrientation)
      {
         if(variation < othVariation)
         {
            phi -= DELTA_CAMERA;
         }
         else
         {
            phi += DELTA_CAMERA;
         }
      }
      else
      {
         if(variation < othVariation)
         {
            phi += DELTA_CAMERA;
         }
         else
         {
            phi -= DELTA_CAMERA;
         }
      }
   }
}

