/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "goal.h"


/********************************************************************
 *                            Constructor                           *
 ********************************************************************/
goal::goal(string fileName)
{
   model = new aniModel();
   model->loadModel(fileName);
   model->calculateBoundingBox(min, max);
}

/********************************************************************
 *                            Destructor                            *
 ********************************************************************/
goal::~goal()
{
   if(model)
   {
      delete(model);
   }
}

/********************************************************************
 *                              render                              *
 ********************************************************************/
void goal::render(camera* gameCamera)
{
   float halfz = FIELD_Z / 2.0;
   float halfx = FIELD_X / 2.0;

   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


   model->loadToGraphicMemory();

   /* Define order of draw (near camera goes last) */
   if(gameCamera->getCameraX() < halfx)
   {
      /* Draw Upper Goal */
      glPushMatrix();
         glTranslatef(GOAL1_X,0,halfz);
         glRotatef(90, 0,1,0);
         model->renderFromGraphicMemory();
      glPopMatrix();

      /* Draw Botton Goal */
      glPushMatrix();
         glTranslatef(GOAL2_X, 0, halfz);
         glRotatef(-90, 0,1,0);
         model->renderFromGraphicMemory();
      glPopMatrix();
   }
   else
   {
      /* Draw Botton Goal */
      glPushMatrix();
         glTranslatef(GOAL2_X, 0, halfz);
         glRotatef(-90, 0,1,0);
         model->renderFromGraphicMemory();
      glPopMatrix();

      /* Draw Upper Goal */
      glPushMatrix();
         glTranslatef(GOAL1_X,0,halfz);
         glRotatef(90, 0,1,0);
         model->renderFromGraphicMemory();
      glPopMatrix();
   }

   model->removeFromGraphicMemory();

   glDisable(GL_BLEND);
}

