#include "team.h"
#include "../etc/util.h"

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../etc/draw.h"
#include "../etc/dirs.h"
#include "../gui/dntfont.h"

#include "../lang/translate.h"

/*************************************************************
 *                       loadTexture                         *
 *************************************************************/
void team::loadTexture(string strFilename, GLuint pId)
{
   dirs dir;

   SDL_Surface* img = IMG_Load(dir.getRealFile(strFilename).c_str());
   if(!img)
   {
      cout << "Can't Open Texture" << strFilename << endl;
      return; 
   }

   loadTexture(img, pId);

   SDL_FreeSurface(img);
}

/*************************************************************
 *                       loadTexture                         *
 *************************************************************/
void team::loadTexture(SDL_Surface* img, GLuint pId)
{
   glBindTexture(GL_TEXTURE_2D, pId);
   glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,img->w,img->h, 
                0, GL_RGB, GL_UNSIGNED_BYTE, img->pixels);
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
   glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
                   GL_LINEAR_MIPMAP_LINEAR );
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);;

   gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, img->w,
                     img->h, GL_RGBA, GL_UNSIGNED_BYTE, 
                     img->pixels );
}

/*************************************************************
 *                   createDiskTexture                       *
 *************************************************************/
void team::createDiskTexture(SDL_Surface* common, int number, int index,
                             int x, int y)
{
   char buf[5];
   dntFont fnt;

   /* Define font and color */
   fnt.defineFont(DNT_FONT_TIMES, 22);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   fnt.defineFontStyle(DNT_FONT_STYLE_BOLD);
   color_Set(0, 0, 0, 255);

   /* Define the number string */
   sprintf(buf, "%d", number);

   /* Write the number */
   SDL_Surface* tmp = SDL_ConvertSurface(common, common->format, SDL_SWSURFACE);
   fnt.write(tmp, x, y-15, buf, x-15, y-15, x+15, y+15);
   loadTexture(tmp, diskTexture[index]);
   SDL_FreeSurface(tmp);
}

/*************************************************************
 *                       Constructor                         *
 *************************************************************/
team::team(string fileName)
{
   dirs dir;
   FILE* file;
   char buffer[512];
   char buffer2[512];
   int i;
   SDL_Surface* img;
   string tmp;
   int x=0,z=0;

   fileName = dir.getRealFile(fileName);
   /* Open File */
   if(!(file = fopen(fileName.c_str(),"r")))
   {
      printf(gettext("Error while opening team file: %s\n"),
             fileName.c_str());
      return;
   }

   /* Get Team Name */
   if(fgets(buffer, sizeof(buffer), file) != NULL)
   {
      teamName = translateDataString(buffer);
   }

   /* Load Team Texture */
   glGenTextures(1,&teamLogo);
   if(fgets(buffer, sizeof(buffer), file) != NULL)
   {
      sscanf(buffer,"%s", buffer2);
      tmp = dir.getRealFile(buffer2 + string("_symbol.png"));
      loadTexture(tmp, teamLogo);

      /* Load GoalKeeper Texture */
      glGenTextures(1,&goalKeeperTexture);
      tmp = dir.getRealFile(string(buffer2) + "_gkeeper.png");
      loadTexture(tmp, goalKeeperTexture);
   }

   srand(SDL_GetTicks());

   /* Load Disk Textures */
   if(fgets(buffer, sizeof(buffer), file) != NULL)
   {
      sscanf(buffer, "%d %d", &x, &z);
      tmp = dir.getRealFile(string(buffer2) + "_disk.png");
      img = IMG_Load(tmp.c_str());
      if(!img)
      {
         cerr << "Can't load disk texture for: " << buffer2 << endl;
         return;
      }   
      glGenTextures(10, diskTexture);
      for(i = 2; i <= 11; i++)
      {
         /* load the texture */
         createDiskTexture(img, i, i-2, x, z);

         /* using the same for to set random init angles */
         disk[i-2].setAngle((360.0 * (rand() / (RAND_MAX + 1.0))));
      }
      SDL_FreeSurface(img);
   }

   /* Load DiskModel */
   if(fgets(buffer, sizeof(buffer), file) != NULL)
   {
      sscanf(buffer,"%s", buffer2);
      tmp = dir.getRealFile(buffer2);
      diskModel = new aniModel();
      diskModel->loadModel(tmp);
      /* Since there aren't animations, the bounding box is static */
      diskModel->calculateBoundingBox(diskMin, diskMax);
   }

   /* Load goal keeper model */
   if(fgets(buffer, sizeof(buffer), file) != NULL)
   {
      sscanf(buffer,"%s", buffer2);
      tmp = dir.getRealFile(buffer2);
      goalKeeperModel = new aniModel();
      goalKeeperModel->loadModel(tmp);
      /* Since there aren't animations, the bounding box is static */
      goalKeeperModel->calculateBoundingBox(goalKeeperMin, goalKeeperMax);
   }

   fclose(file);
}

/*************************************************************
 *                        Destructor                         *
 *************************************************************/
team::~team()
{
   glDeleteTextures(1,&teamLogo);
   glDeleteTextures(1,&goalKeeperTexture);
   glDeleteTextures(10, diskTexture);
   delete(diskModel);
   delete(goalKeeperModel);
}

/*************************************************************
 *                  startPositionAtField                     *
 *************************************************************/
void team::startPositionAtField(bool upper, bool withBall)
{
   float halfz = FIELD_Z / 2.0;
   float halfx = FIELD_X / 2.0;
   
   int m = 1; //The multiplier to the position if is upper or not.

   /* Set Goal Keeper */
   if(upper)
   {
      goalKeeper.setPosition(FIELD_X - FIELD_SIDE_DELTA - 10,halfz);
      goalKeeper.setAngle(-90);
      m = 1;
   }
   else
   {
      goalKeeper.setPosition(FIELD_SIDE_DELTA + 10,halfz);
      goalKeeper.setAngle(90);
      m = -1;
   }

   /* Set Players */
   if(!withBall)
   {
      /* 2 FWD at sides */
      disk[0].setPosition(halfx+m*12, FIELD_SIDE_DELTA + 12);
      disk[4].setPosition(halfx+m*12, FIELD_Z - FIELD_SIDE_DELTA - 12);

      /* Center FWD */
      disk[8].setPosition(halfx+m*75, halfz);

      /* 2 Midfields */
      disk[9].setPosition(halfx+m*12, halfz+75);
      disk[7].setPosition(halfx+m*12, halfz-75);

      /* 5 Defenders */
      disk[5].setPosition(halfx+m*110, halfz+65);
      disk[6].setPosition(halfx+m*110, halfz-65);
      disk[3].setPosition(halfx+m*140, halfz);
      disk[1].setPosition(halfx+m*195, halfz-50);
      disk[2].setPosition(halfx+m*195, halfz+50);
   }
   else
   {
      /* 2 FWD at sides */
      disk[0].setPosition(halfx+m*12, FIELD_SIDE_DELTA + 12);
      disk[4].setPosition(halfx+m*12, FIELD_Z - FIELD_SIDE_DELTA - 12);

      /* 2 Midfields and 1 FWD inner circle */
      disk[8].setPosition(halfx+m*20, halfz-22);
      disk[9].setPosition(halfx+m*12, halfz+10);
      disk[7].setPosition(halfx+m*40, halfz+38);

      /* 5 Defenders */
      disk[5].setPosition(halfx+m*110, halfz+65);
      disk[6].setPosition(halfx+m*110, halfz-65);
      disk[3].setPosition(halfx+m*140, halfz);
      disk[1].setPosition(halfx+m*195, halfz-50);
      disk[2].setPosition(halfx+m*195, halfz+50);
   }
}

/*************************************************************
 *                          render                           *
 *************************************************************/
void team::render()
{
   int i;
   /* Render the goal Keeper */
   goalKeeperModel->loadToGraphicMemory(false);
   glBindTexture(GL_TEXTURE_2D, goalKeeperTexture);

   glPushMatrix();
      glTranslatef(goalKeeper.getPosX(), 0, goalKeeper.getPosZ());
      glRotatef(goalKeeper.getAngle(), 0,1,0);
      goalKeeperModel->renderFromGraphicMemory();
   glPopMatrix();

   goalKeeperModel->removeFromGraphicMemory();

   /* Render the disks */
   diskModel->loadToGraphicMemory(false);
   for(i=0; i < 10; i++)
   {
     glBindTexture(GL_TEXTURE_2D, diskTexture[i]);
      glPushMatrix();
         glTranslatef(disk[i].getPosX(), 0, disk[i].getPosZ());
         glRotatef(disk[i].getAngle(), 0,1,0);
         diskModel->renderFromGraphicMemory();
      glPopMatrix();
   }
   diskModel->removeFromGraphicMemory();
}

/*************************************************************
 *                          render                           *
 *************************************************************/
void team::render(float pos[11][2], float angles[11])
{
   int i;

   /* Render the goal Keeper */
   goalKeeperModel->loadToGraphicMemory(false);
   glBindTexture(GL_TEXTURE_2D, goalKeeperTexture);
   glPushMatrix();
      glTranslatef(pos[0][0], 0, pos[0][1]);
      glRotatef(angles[0], 0,1,0);
      goalKeeperModel->renderFromGraphicMemory();
   glPopMatrix();
   goalKeeperModel->removeFromGraphicMemory();
   /*goalKeeperModel->renderBoundingBox(goalKeeperMin, goalKeeperMax, 
                                      pos[0][0], 0.0, pos[0][1],
                                      angles[0]);*/

   /* Render the disks */
   diskModel->loadToGraphicMemory(false);
   for(i=0; i < 10; i++)
   {
      glBindTexture(GL_TEXTURE_2D, diskTexture[i]);
      glPushMatrix();
         glTranslatef(pos[i+1][0], 0, pos[i+1][1]);
         glRotatef(angles[i+1], 0,1,0);
         diskModel->renderFromGraphicMemory();
      glPopMatrix();
   }
   diskModel->removeFromGraphicMemory();
}

/*************************************************************
 *                     getGoalKeeper                         *
 *************************************************************/
teamPlayer* team::getGoalKeeper()
{
   return(&goalKeeper);
}

/*************************************************************
 *                    getNearestPlayer                       *
 *************************************************************/
teamPlayer* team::getNearestPlayer(float x, float z)
{
   teamPlayer* res = &disk[0];
   float minDist = sqrt( (res->getPosX() - x) * 
                         (res->getPosX() - x) +
                         (res->getPosZ() - z) * 
                         (res->getPosZ() - z) );
   float dist;
   int i;

   for(i=1; i<10; i++)
   {
      /* Calculate distance */
      dist = sqrt( (disk[i].getPosX() - x) *
                   (disk[i].getPosX() - x) +
                   (disk[i].getPosZ() - z) *
                   (disk[i].getPosZ() - z) );

      /* Swap it, if is lesser than the current */
      if(dist < minDist)
      {
         minDist = dist;
         res = &disk[i];
      }
   }

   return(res);
}

/*************************************************************
 *                       selectPlayer                        *
 *************************************************************/
teamPlayer* team::selectPlayer(float x, float z)
{
   int i;
   teamPlayer* res = NULL;

   float mouseMin[3];
   float mouseMax[3];

   float minObj[3],
         maxObj[3];

   GLfloat X[4]; GLfloat Z[4];
   X[0] = diskMin[0];
   Z[0] = diskMin[2];
   X[1] = diskMin[0];
   Z[1] = diskMax[2];
   X[2] = diskMax[0];
   Z[2] = diskMax[2];
   X[3] = diskMax[0];
   Z[3] = diskMin[2];

   /* Create Mouse Bounding Box */
   mouseMin[0] = x - 1;
   mouseMin[1] = 0;
   mouseMin[2] = z - 1;
   mouseMax[0] = x + 1;
   mouseMax[1] = 1;
   mouseMax[2] = z + 1;

   for(i=0; ( (i < 10) && (res == NULL) ); i++)
   {
      /* Calculate the real Disk Bounding Box. */
      rotTransBoundingBox(disk[i].getAngle(), X, Z, disk[i].getPosX(), 0.0, 
                          0.0, disk[i].getPosZ(), minObj, maxObj);
      if(intercepts(minObj, maxObj, mouseMin, mouseMax, 1))
      {
         /* Selected the object */
         res = &disk[i];
      }
   }
   return(res);
}

/*************************************************************
 *                       getTeamLogo                         *
 *************************************************************/
GLuint team::getTeamLogo()
{
   return(teamLogo);
}

/*************************************************************
 *                         getTeamName                       *
 *************************************************************/
string team::getTeamName()
{
   return(teamName);
}


