/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "teamplayer.h"

/******************************************************************
 *                           Constructor                          *
 ******************************************************************/
teamPlayer::teamPlayer()
{
   posX = 0;
   posZ = 0;
   prvX = 0;
   prvZ = 0;
   angle = 0;
   dX = 0;
   dZ = 0;
   velocity = 0;
   velUpdate = false;
}

/******************************************************************
 *                            Destructor                          *
 ******************************************************************/
teamPlayer::~teamPlayer()
{
}

/******************************************************************
 *                             getPosX                            *
 ******************************************************************/
float teamPlayer::getPosX()
{
   return(posX);
}

/******************************************************************
 *                             getPosZ                            *
 ******************************************************************/
float teamPlayer::getPosZ()
{
   return(posZ);
}

/******************************************************************
 *                            getAngle                            *
 ******************************************************************/
float teamPlayer::getAngle()
{
   return(angle);
}

/******************************************************************
 *                           getVelocity                          *
 ******************************************************************/
float teamPlayer::getVelocity()
{
   return(velocity);
}

/******************************************************************
 *                           getOrientX                           *
 ******************************************************************/
float teamPlayer::getOrientX()
{
   return(dX);
}

/******************************************************************
 *                           getOrientZ                           *
 ******************************************************************/
float teamPlayer::getOrientZ()
{
   return(dZ);
}

/******************************************************************
 *                            setAngle                            *
 ******************************************************************/
void teamPlayer::setAngle(float alpha)
{
   angle = alpha;
}

/******************************************************************
 *                           setPosition                          *
 ******************************************************************/
void teamPlayer::setPosition(float x, float z)
{
   posX = x;
   posZ = z;
}

/******************************************************************
 *                           setVelocity                          *
 ******************************************************************/
void teamPlayer::setVelocity(float x, float z, float vel)
{
   dX = x;
   dZ = z;
   velocity = vel;
}

/******************************************************************
 *                         setNextVelocity                        *
 ******************************************************************/
void teamPlayer::setNextVelocity(float x, float z, float vel)
{
   velUpdate = true;
   nextDX = x;
   nextDZ = z;
   nextVelocity = vel;
}

/******************************************************************
 *                         updateVelocity                         *
 ******************************************************************/
void teamPlayer::updateVelocity()
{
   if(velUpdate) 
   {
      velUpdate = false;
      dX = nextDX;
      dZ = nextDZ;
      velocity = nextVelocity;
   }
}

/******************************************************************
 *                             goBack                             *
 ******************************************************************/
void teamPlayer::goBack()
{
   posX = prvX;
   posZ = prvZ;
}

/******************************************************************
 *                         updatePosition                         *
 ******************************************************************/
bool teamPlayer::updatePosition(float friction)
{
   /* Update velocity to externally set new velocity (if needed) */
   updateVelocity();
   
   /* Save previous position */
   prvX = posX;
   prvZ = posZ;
   
   /* Apply velocity */
   posX += velocity*dX;
   posZ += velocity*dZ;
   
   /* Check field side collision */
   checkFieldCollision();
   
   /* Apply Friction */
   if(velocity > 0)
   {
      velocity -= friction;
      if(velocity < 0)
      {
         velocity = 0;
      }
   }
   else if(velocity < 0)
   {
      velocity += friction;
      if(velocity > 0)
      {
         velocity = 0;
      }
   }

   return(velocity != 0);
}

/******************************************************************
 *                       checkFieldCollision                      *
 ******************************************************************/
void teamPlayer::checkFieldCollision(){
   if (posX > FIELD_X)
   {  
      posX = FIELD_X;
      dX = -dX;
   }
   else if (posX < 0)
   {  
      posX = 0;
      dX = -dX;
   }
   
   if (posZ > FIELD_Z)
   {  
      posZ = FIELD_Z;
      dZ = -dZ;
   }
   else if (posZ < 0)
   {  
      posZ = 0;
      dZ = -dZ;
   }
}
