#ifndef _teamPlayer_h
#define _teamPlayer_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "field.h"
#include "fobject.h"

/*! The teamPlayer is the representation of teamPlayers and goal keepers.
 * Each team has 1 goal keeper and 10 teamPlayers to play with;*/
class teamPlayer: public fobject
{
   public:
      /*! Constructor */
      teamPlayer();

      /*! Destructor */
      ~teamPlayer();

      /*! Get the teamPlayer X Position
       * \return teamPlayer X position */
      float getPosX();
      /*! Get the teamPlayer Z Position
       * \return teamPlayer Z position */
      float getPosZ();

      /*! Get the teamPlayer angle
       * \return -> current teamPlayer orientation angle */
      float getAngle();
   
      /*! Get the teamPlayer velocity
       * \return -> current teamPlayer velocity */
      float getVelocity();
      
      /*! Get the teamPlayer X orientation
       * \return x component of orientation vector */
       float getOrientX();

      /*! Get the teamPlayer Z orientation
       * \return z component of orientation vector */
       float getOrientZ();

      /*! Set the teamPlayer orientation angle
       * \alpha -> new angle value */
      void setAngle(float alpha);

      /*! Set the teamPlayer new position
       * \param x -> new x position
       * \param z -> new z position */
      void setPosition(float x, float z);

      /*! Set the teamPlayer position to the previous position */
      void goBack();

      /*! Update the Position of the player, based on its velocity
       * \param friction -> friction value used.
       * \return -> true if the player move, false if is stable */
      bool updatePosition(float friction);

      /*! Set the player velocity
       * \param x -> X velocity component
       * \param z -> Z velocity component 
       * \param vel -> velocity value */
      void setVelocity(float x, float z, float vel);
      
      /*! Set the player velocity
       * \param x -> next X velocity component
       * \param z -> next Z velocity component 
       * \param vel -> next velocity value */
      void setNextVelocity(float x, float z, float vel);
      
      /*! Update velocity with 'next velocity' values*/
      void updateVelocity();
      
   protected:
      float angle;      /**< Current Disk Angle Orientation */
      float posX,       /**< Disk X Position */
            posZ;       /**< Disk Z Position */
      float prvX,       /**< Disk previous X position */
            prvZ;       /**< Disk previous Z position */

      float dX,        /**< X coordinate velocity */
            dZ;        /**< Z coordinate velocity */
      float velocity;  /**< Velocity */
   
      float nextDX,        /**< Next X coordinate velocity */
            nextDZ;        /**< Next Z coordinate velocity */
      float nextVelocity;  /**< Next Velocity */
      bool velUpdate;      /**< Flag indicating if velocity must be updated*/
   
      /*! Checks if teamplayer is outside of the field. If so, it's moved back 
       *  to the field limits and its direction is changed */
      void checkFieldCollision();
   
};


#endif


