#ifndef _xut_h
#define _xut_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "team.h"
#include "camera.h"
#include "../etc/util.h"
#include "ball.h"
#include "goal.h"
#include "rules.h"
#include "replay.h"
#include "../physics/forceio.h"
#include "../physics/collision.h"
#include "../gui/initial.h"
#include "../gui/guiscore.h"
#include "../gui/guistatus.h"
#include "../gui/guimessage.h"
#include "../gui/maingui.h"
#include "../gui/optscreen.h"
#include "../etc/cursor.h"
#include "../sound/sound.h"

#define XUT_STATE_NORMAL                 0
#define XUT_STATE_GOAL_KEEPER_POSITION   1
#define XUT_STATE_DISK_POSITION          2

/*! xut is an implementation of the button football game. */
class xut
{
   public:
      /*! Constructor
       * \param screen -> current used screen window
       * \param scWidth -> current screen width
       * \param scHeight -> current screen height */
      xut(SDL_Surface* screen, int scWidth, int scHeight);
      /*! Destructor */
      ~xut();

      /*! Run the xut game */
      void run();

   protected:
      /*! Define Screen to actual Screen Size  
       * \param screen -> Pointer to the Screen's Surface
       * \param actualFarView -> FarView Value */
      void redefineWindow(SDL_Surface *screen, int actualFarView);

      /*! Update the mouse world position at floor (table) */
      void updateMouseFloorPos();

      /*! Render the game */
      void render();

      /*! Render the graphic user interface */
      void renderGUI();

      /*! Verify Game Input Events */
      void gameIO();

      /*! Verify input for the normal game */
      void normalGameIO();

      /*! Verify the input for goal keeper position definition */
      void goalKeeperPositionIO();

      /*! Verify input for disk position definition */
      void diskPositionIO();

      /*! Verify the Result of the rules */
      void verifyRulesResult();

      /*! Load Splash Screen */
      void splashScreen();

      /*! Fade in Some Texture on screen */
      void fadeInTexture(GLuint id, int x1, int y1, int x2, int y2,
            int sizeX, int sizeY);

      /*! Fade Out Some texture on screen */
      void fadeOutTexture(GLuint id, int x1, int y1, int x2, int y2,
            int sizeX, int sizeY);

      /*! Load the Background Texture */
      void loadBackgroundTexture();

      /*! Do the main Loop */
      void mainLoop();

      /*! Tell that will do a shoot */
      void prepareToShoot();

      int state;                    /**< The xut State */

      guiScore gameScore;           /**< The Game Score */
      guiStatus gameStatus;         /**< The Game Status */
      guiMessage gameMessage;       /**< The Game Message */
      mainGui* gameMainGui;         /**< The main gui used */

      cursor* cursors;              /**< The mouse cursors used */

      options opt;                  /**< Current Options */

      camera* gameCamera;           /**< The camera used */
      sound snd;                    /*< The sound controller */

      GLuint bgTexture;             /**< BackTexture for Initial Screens */

      field* gameField;             /**< The game field */
      team* teamA;                  /**< Team for the player A */
      team* teamB;                  /**< Team for the player B */
      ball* gameBall;               /**< The game ball used */

      bool exit;                    /**< Flux Control variable */

      bool initedTurn;              /**< If already inited the turn */

      goal* goals;                  /**< The field Goals */

      forceIO force;                /**< The force controller */
      teamPlayer* selectedPlayer;   /**< Current selected player */

      collision physic;             /**< The Physics Collision Controller */

      rules* rulesSystem;           /**< The Rules Controller */
      replay* replayer;             /**< The Replay Controller */

      Uint8 *keys;                  /**< current SDL key state */
      int mouseX,                   /**< current mouse X position */
          mouseY;                   /**< current mouse Y position */
      Uint8 mButton;                /**< current mouse button state */
      GLdouble xReal,               /**< mouse WORLD X coordinate */
               zReal;               /**< mouse WORLD Z coordinate */

      SDL_Surface* actualScreen;    /**< current window screen surface */
      int screenWidth,              /**< current screen width */
          screenHeight;             /**< current screen height */

      GLdouble proj[16];            /**< Projection Matrix */
      GLdouble modl[16];            /**< ModelView Matrix  */
      GLint viewPort[4];            /**< ViewPort Matrix */
};


#endif

