#ifndef _xut_options_h
#define _xut_options_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <string>
using namespace std;


#define XUT_OPTIONS_CAMERA_TYPE_STATIC         0
#define XUT_OPTIONS_CAMERA_TYPE_FOLLOW_FOULS   1
#define XUT_OPTIONS_CAMERA_TYPE_FOLLOW_BALL    2

#define XUT_OPTIONS_FIELD_DETAIL_LOW           0
#define XUT_OPTIONS_FIELD_DETAIL_NORMAL        1
#define XUT_OPTIONS_FIELD_DETAIL_HIGH          2



#define XUT_TOTAL_KEYS              14

#define XUT_KEY_CAMERA_UP            0
#define XUT_KEY_CAMERA_DOWN          1
#define XUT_KEY_CAMERA_LEFT          2
#define XUT_KEY_CAMERA_RIGHT         3
#define XUT_KEY_CAMERA_ROTATE_LEFT   4
#define XUT_KEY_CAMERA_ROTATE_RIGHT  5
#define XUT_KEY_CAMERA_ROTATE_UP     6
#define XUT_KEY_CAMERA_ROTATE_DOWN   7
#define XUT_KEY_CAMERA_ZOOM_UP       8
#define XUT_KEY_CAMERA_ZOOM_DOWN     9

#define XUT_KEY_WILL_SHOOT          10
#define XUT_KEY_REPLAY              11

#define XUT_KEY_ROTATE_LEFT         12
#define XUT_KEY_ROTATE_RIGHT        13

#define XUT_LANGUAGE_EN              0
#define XUT_LANGUAGE_PT_BR           1
#define XUT_LANGUAGE_CA              2
#define XUT_LANGUAGE_ES              3
#define XUT_LANGUAGE_DA              4
#define XUT_LANGUAGE_IT              5

/*! The options class keep some optional informations */
class options
{
   public:
      /*! Load options from file */
      void load();
      /*! Save options to file */
      void save();
 
      /*! Get AntiAliasing Value
       * \return -> current antiAliasing */
      int getAntiAliasing();
      /*! Set Next AntiAliasing Value */
      void setAntiAliasing();
      
      /*! Get the current window Width
       * \return -> current window width */
      int getWindowX();
      /*! Set the current window Height
       * \return -> current window height */
      int getWindowY();
      /*! Set the next available window size */
      void setWindowSize();
      /*! Get resolutions information */
      void defineResolutions();
      /*! Get the potential (desired after options chage) window size 
       * \param wX -> window width
       * \param wY -> window height */
      void getPotentialWindowSize(int& wX, int& wY);
      
      /*! Verify if is to set fullscreen
       * \return -> true if fullScreen */
      bool getFullScreen();
      /*! Define if will fullScreen or not */
      void setFullScreen();

      /*! Get the fieldDetail
       * \return -> current fieldDetail */
      int getFieldDetail();
      /*! Set the fieldDetail to the next avaiable option */
      void setFieldDetail();

      /*! Get sound volume
       * \return -> current sound volume */
      int getSoundVolume();
      /*! Set soundVolume += 51 */
      void setSoundVolume();

      /*! Get the camera type
       * \return -> cameraType constant */
      int getCameraType();
      /*! Set the camera type */
      void setCameraType();

      /*! Get the desired key value
       * \param key -> XUT_KEY constant
       * \return -> key value map to the desired key */
      int getKey(int key);
      /*! Set the keymap
       * \param key -> key code
       * \param value -> new keycode to map as the key */
      void setKey(int key, int value);

      /*! Set the language to lNumber
       * \param lNumber -> number of the language to use */
      void setLanguage(int lNumber);
      /*! Get current language
       * \return number of the current language */
      int getLanguage();
      /*! Get the current language name
       * \return string with current language name */
      string getLanguageName();
      /*! Set the current locale and language */
      void setLocale();
 
   protected:

      /* Set the keys as the default ones */
      void setDefaultKeys();

      static int keys[XUT_TOTAL_KEYS];  /**< The Key Map */
      static int antiAliasing;  /**< AntiAliasing Value */
      static int windowX;       /**< Window Width */
      static int windowY;       /**< Window Height */
      static int resolutionId;  /**< Resolution positon on vector */
      static bool fullScreen;   /**< FullScreen Toggle */
      static int fieldDetail;   /**< Field Level of Detail */
      static int soundVolume;   /**< Sound Volume */
      static int cameraType;    /**< Camera Type constant */
      static int language;      /**< Language Number */
      static SDL_Rect** resolutions;    /**< the avaible resolutions */ 
};

#endif

