/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "guimessage.h"
#include "dntfont.h"

#include "../etc/draw.h"
#include "../etc/dirs.h"

#define GUI_MESSAGE_ALPHA_DELTA  0.005

/*********************************************************************
 *                           Constructor                             *
 *********************************************************************/
guiMessage::guiMessage()
{
}

/*********************************************************************
 *                            Destructor                             *
 *********************************************************************/
guiMessage::~guiMessage()
{
}

/*********************************************************************
 *                               init                                *
 *********************************************************************/
void guiMessage::init(string imageFile)
{
   dirs dir;

   if(image)
   {
      /* Already inited! */
      return; 
   }

   /* Load Message */
   image = IMG_Load(dir.getRealFile(imageFile).c_str());
   if(!image)
   {
      printf(gettext("Couldn't load texture: %s\n"), imageFile.c_str());
   }

   /* Create OpenGL Texture  */
   glGenTextures(1, &texture);
}

/*********************************************************************
 *                               quit                                *
 *********************************************************************/
void guiMessage::quit()
{
   if(image)
   {
      SDL_FreeSurface(image);
      image = NULL;
      glDeleteTextures(1, &texture);
   }
}

/*********************************************************************
 *                               set                                 *
 *********************************************************************/
void guiMessage::set(string msg)
{
   dntFont fnt;
   SDL_Surface* tmp;

   /* Verify if image is defined */
   if(!image)
   {
      return;
   }

   /* Define Machine Bit Order */
   Uint32 rmask, gmask, bmask, amask;
   #if SDL_BYTEORDER == SDL_BIG_ENDIAN
      rmask = 0xff000000;
      gmask = 0x00ff0000;
      bmask = 0x0000ff00;
      amask = 0x000000ff;
   #else
      rmask = 0x000000ff;
      gmask = 0x0000ff00;
      bmask = 0x00ff0000;
      amask = 0xff000000;
   #endif

   /* Define Font */
   fnt.defineFont(DNT_FONT_ARIAL, 10);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   fnt.defineFontStyle(DNT_FONT_STYLE_NORMAL);

   /* Create temp */
   tmp = SDL_ConvertSurface(image, image->format, 
                            SDL_SWSURFACE);

   /* Draw the Text */
   color_Set(255, 255, 255, 255);
   fnt.write(tmp,0, 10, msg, 0, 0, tmp->w-1, tmp->h-1);

   setTextureRGBA(tmp, texture);
   alpha = 1.0;

   /* Delete the no more needed surface */
   SDL_FreeSurface(tmp);
}

/*********************************************************************
 *                               draw                                *
 *********************************************************************/
void guiMessage::draw()
{
   if( (!image) || (alpha <= 0.0) )
   {
      /* If not load image or alpha NULL, so not need to draw */
      return;
   }

   float hX = image->w / 2.0;
   float hY = image->h / 2.0;


   glColor4f(1.0,1.0,1.0,alpha);
   alpha -= GUI_MESSAGE_ALPHA_DELTA;
   if(alpha < 0.0)
   {
      alpha = 0.0;
   }

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);

   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, texture);

   glPushMatrix();
      glTranslatef(hX, opt.getWindowY() - (hY), 0.0);
      glBegin(GL_QUADS);
         glTexCoord2d(0,1);
         glVertex2f(-hX,-hY);
         glTexCoord2d(0,0);
         glVertex2f(-hX,hY);
         glTexCoord2d(1,0);
         glVertex2f(hX,hY);
         glTexCoord2d(1,1);
         glVertex2f(hX,-hY);
      glEnd();
   glPopMatrix();

   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);

   glColor4f(1.0,1.0,1.0,1.0);
}

/*********************************************************************
 *                         static members                            *
 *********************************************************************/
SDL_Surface* guiMessage::image = NULL;
GLuint guiMessage::texture = 0;
GLfloat guiMessage::alpha=1.0;

