/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "guiscore.h"
#include "dntfont.h"

#include "../etc/draw.h"
#include "../etc/dirs.h"

/***************************************************************
 *                         Constructor                         *
 ***************************************************************/
guiScore::guiScore()
{
}

/***************************************************************
 *                          Destructor                         *
 ***************************************************************/
guiScore::~guiScore()
{
}

/***************************************************************
 *                            quit                             *
 ***************************************************************/
void guiScore::quit()
{
   if(scoreAlloc)
   {
      glDeleteTextures(1, &scoreTexture);
   }
}

/***************************************************************
 *                            reset                            *
 ***************************************************************/
void guiScore::reset(GLuint logoA, GLuint logoB)
{
   /* Create the draw Texture and Surface */
   if(!scoreAlloc)
   {
      glGenTextures(1,&scoreTexture);
      scoreAlloc = true;
   }

   teamLogoA = logoA;
   teamLogoB = logoB;
   teamGoalsA = 0;
   teamGoalsB = 0;
   
   setTextTexture();
}

/***************************************************************
 *                         getTeamScore                        *
 ***************************************************************/
void guiScore::getTeamScore(int& goalsA, int& goalsB)
{
   goalsA = teamGoalsA;
   goalsB = teamGoalsB;
}

/***************************************************************
 *                          goalTeamA                          *
 ***************************************************************/
void guiScore::goalTeamA()
{
   teamGoalsA++;
   setTextTexture();
}

/***************************************************************
 *                          goalTeamB                          *
 ***************************************************************/
void guiScore::goalTeamB()
{
   teamGoalsB++;
   setTextTexture();
}

/***************************************************************
 *                       setTextTexture                        *
 ***************************************************************/
void guiScore::setTextTexture()
{

   /* Define Machine Bit Order */
   Uint32 rmask, gmask, bmask, amask;
   #if SDL_BYTEORDER == SDL_BIG_ENDIAN
      rmask = 0xff000000;
      gmask = 0x00ff0000;
      bmask = 0x0000ff00;
      amask = 0x000000ff;
   #else
      rmask = 0x000000ff;
      gmask = 0x0000ff00;
      bmask = 0x00ff0000;
      amask = 0xff000000;
   #endif

   /* Define the Score Text */
   char text[16];
   sprintf(text,"%d - %d", teamGoalsA, teamGoalsB);

   /* Set the desired font */
   dntFont fnt;
   fnt.defineFont(DNT_FONT_ARIAL, 40);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   fnt.defineFontStyle(DNT_FONT_STYLE_NORMAL);

   /* Create the Surface to write to */
   SDL_Surface* scoreImg;
   scoreImg = SDL_CreateRGBSurface(SDL_SWSURFACE, 128, 64, 32,
                                    rmask, gmask, bmask, amask);
   //SDL_FillRect(scoreImg, NULL, SDL_MapRGBA(scoreImg->format, 0, 0, 0, 0));

   /*Uint32 colorkey = SDL_MapRGBA(scoreImg->format, 0, 0, 0, 0);
   SDL_SetColorKey(scoreImg, SDL_SRCCOLORKEY, colorkey);*/

   /* Write text to Surface */
   color_Set(255, 255, 255, 255);
   fnt.write(scoreImg,0, 10, text, 0, 0, 127, 63, true); 

   //rectangle_Fill(scoreImg, 10,10,80,50);
   
   /* Set Texture */
   setTextureRGBA(scoreImg, scoreTexture);

   /* Free the temporary surface */
   SDL_FreeSurface(scoreImg);

}

/***************************************************************
 *                           drawQuad                          *
 ***************************************************************/
void guiScore::drawQuad(int sizeX, int sizeY)
{
   glBegin(GL_QUADS);
      glTexCoord2f(0,1);
      glVertex2f(-sizeX, -sizeY);
	
      glTexCoord2f(1,1);
      glVertex2f(sizeX, -sizeY);
	
      glTexCoord2f(1,0);
      glVertex2f(sizeX, sizeY);
	
      glTexCoord2f(0,0);
      glVertex2f(-sizeX, sizeY);
   glEnd();
}

/***************************************************************
 *                             draw                            *
 ***************************************************************/
void guiScore::draw()
{
   int screenX = opt.getWindowX();
   int screenY = opt.getWindowY();

   glEnable(GL_TEXTURE_2D);
   glEnable(GL_BLEND);
  
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
   glBindTexture(GL_TEXTURE_2D, scoreTexture);

   glPushMatrix();
      glTranslatef(screenX - 80, screenY-16, 0);
      drawQuad(32,16);
   glPopMatrix();

   glBindTexture(GL_TEXTURE_2D, teamLogoA);
   glPushMatrix();
      glTranslatef(screenX - 128, screenY-16, 0);
      drawQuad(16,16);
   glPopMatrix();
   
   glBindTexture(GL_TEXTURE_2D, teamLogoB);
   glPushMatrix();
      glTranslatef(screenX - 32, screenY-16, 0);
      drawQuad(16,16);
   glPopMatrix();


   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
}

/***************************************************************
 *                       Static Members                        *
 ***************************************************************/
GLuint       guiScore::teamLogoA = 0;
GLuint       guiScore::teamLogoB = 0;
int          guiScore::teamGoalsA = 0;
int          guiScore::teamGoalsB = 0;
bool         guiScore::scoreAlloc = false;
GLuint       guiScore::scoreTexture;

