#ifndef _guiscore_h
#define _guiscore_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "../etc/options.h"

/*! The guiScore Class will count each team goals and display it */
class guiScore
{
   public:
      /*! Constructor */
      guiScore();
      /*! Destructor */
      ~guiScore();

      /*! Reset the guiScore
       * \param logoA -> logo for teamA
       * \param logoB -> logo for teamB */
      void reset(GLuint logoA, GLuint logoB);

      /*! Quit the guiScore
       * \note -> must be called before quit the program. */
      void quit();

      /*! Draw the logo to the screen
       * \note -> projection must be 2D before this call */
      void draw();

      /*! Get total score for each teams
       * \param goalsA -> total goals done by teamA
       * \param goalsB -> total goals done by teamB */
      void getTeamScore(int& goalsA, int& goalsB);

      /*! Called when Goal for teamA occurs */
      void goalTeamA();

      /*! Called when Goal for teamB occurs */
      void goalTeamB();

   protected:
      /*! The the score text Texture */
      void setTextTexture();

      /*! Draw a quad */
      void drawQuad(int sizeX, int sizeY);

      options opt;

      static GLuint teamLogoA;  /**< Current Logo for teamA */
      static GLuint teamLogoB;  /**< Current Logo for teamB */

      static GLuint scoreTexture; /**< The Score Texture */
      static bool scoreAlloc;     /**< If alloc the score texture */

      static int teamGoalsA;    /**< Current Goals for teamA */
      static int teamGoalsB;    /**< Current Goals for teamB */
};

#endif

