
/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "guistatus.h"
#include "dntfont.h"
#include "../etc/draw.h"

/***********************************************************************
 *                              Constructor                            *
 ***********************************************************************/
guiStatus::guiStatus()
{
}

/***********************************************************************
 *                               Destructor                            *
 ***********************************************************************/
guiStatus::~guiStatus()
{
}

/***********************************************************************
 *                                 quit                                *
 ***********************************************************************/
void guiStatus::quit()
{
   if(statusAlloc)
   {
      glDeleteTextures(1, &statusTexture);
   }
}

/***********************************************************************
 *                             setActiveTeam                           *
 ***********************************************************************/
void guiStatus::setActiveTeam(GLuint tlogo)
{
   activeTeamLogo = tlogo;
}

/***********************************************************************
 *                            setGlobalTouches                         *
 ***********************************************************************/
void guiStatus::setGlobalTouches(int touches)
{
   /* Define Machine Bit Order */
   Uint32 rmask, gmask, bmask, amask;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
   rmask = 0xff000000;
   gmask = 0x00ff0000;
   bmask = 0x0000ff00;
   amask = 0x000000ff;
#else
   rmask = 0x000000ff;
   gmask = 0x0000ff00;
   bmask = 0x00ff0000;
   amask = 0xff000000;
#endif

   /* Define the Staus Text */
   char text[256];
   sprintf(text,gettext("Global Touches: %d"), touches);

   /* Set the desired font */
   dntFont fnt;
   fnt.defineFont(DNT_FONT_ARIAL, 12);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   fnt.defineFontStyle(DNT_FONT_STYLE_NORMAL);

   /* Create the Surface to write to */
   SDL_Surface* scoreImg;
   scoreImg = SDL_CreateRGBSurface(SDL_SWSURFACE, 128, 64, 32,
                                   rmask, gmask, bmask, amask);

   /* Write text to Surface */
   color_Set(255, 255, 255, 255);
   fnt.write(scoreImg, 0, 5, gettext("Turn:"), 0, 0, 127, 63, true);
   fnt.write(scoreImg, 0, 30, text, 0, 0, 127, 63, true);

   /* Set the Texture */
   setTextureRGBA(scoreImg, statusTexture);

   /* Free the surface */
   SDL_FreeSurface(scoreImg);
}

/***********************************************************************
 *                                 draw                                *
 ***********************************************************************/
void guiStatus::draw()
{
   int screenX = opt.getWindowX();

   glEnable(GL_TEXTURE_2D);
   glEnable(GL_BLEND);

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

   /* Draw Text */
   glBindTexture(GL_TEXTURE_2D, statusTexture);
   glPushMatrix();
      glTranslatef(screenX - 80, 35, 0);
      glBegin(GL_QUADS);
        glTexCoord2f(0,1);
        glVertex2f(-64, -32);

        glTexCoord2f(1,1);
        glVertex2f(64, -32);

        glTexCoord2f(1,0);
        glVertex2f(64, 32);

        glTexCoord2f(0,0);
        glVertex2f(-64, 32);
      glEnd();
   glPopMatrix();

   /* Draw Act Team Logo */
   glBindTexture(GL_TEXTURE_2D, activeTeamLogo);
   glPushMatrix();
      glTranslatef(screenX - 40, 55, 0);
      glBegin(GL_QUADS);
        glTexCoord2f(0,1);
        glVertex2f(-16, -16);

        glTexCoord2f(1,1);
        glVertex2f(16, -16);

        glTexCoord2f(1,0);
        glVertex2f(16, 16);

        glTexCoord2f(0,0);
        glVertex2f(-16, 16);
      glEnd();
   glPopMatrix();

   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
}

/***********************************************************************
 *                            Static Members                           *
 ***********************************************************************/
GLuint guiStatus::activeTeamLogo;
GLuint guiStatus::statusTexture;  
bool guiStatus::statusAlloc = false;

