
/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "initial.h"
#include "../etc/draw.h"
#include "../lang/translate.h"
#include "dntfont.h"

#include <iostream>
using namespace std;

#define BG_SIZE_X 800
#define BG_SIZE_Y 600

/********************************************************************
 *                             Constructor                          *
 ********************************************************************/
initial::initial(GLdouble* proj,GLdouble* modl, GLint* vPort, GLuint bg)
{
   current = -1;

   projection = proj;
   modelView = modl;
   viewPort = vPort;
   bgTexture = bg;

   loadTextTextures();
}

/********************************************************************
 *                              Destructor                          *
 ********************************************************************/
initial::~initial()
{
   glDeleteTextures(INITIAL_EXIT+1, textTexture);
}

/********************************************************************
 *                         loadTextTexture                          *
 ********************************************************************/
void initial::loadTextTextures()
{
   /* Define Machine Bit Order */
   Uint32 rmask, gmask, bmask, amask;
   #if SDL_BYTEORDER == SDL_BIG_ENDIAN
      rmask = 0xff000000;
      gmask = 0x00ff0000;
      bmask = 0x0000ff00;
      amask = 0x000000ff;
   #else
      rmask = 0x000000ff;
      gmask = 0x0000ff00;
      bmask = 0x00ff0000;
      amask = 0xff000000;
   #endif

   /* Generate All Textures */
   glGenTextures(INITIAL_EXIT+1, textTexture);

   /* Set Font */
   dntFont fnt;
   fnt.defineFont(DNT_FONT_ARIAL, 26);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   SDL_Surface* image;
   
   /* Load the textures */
   int i;
   for(i = 0; i <= INITIAL_EXIT; i++)
   {
      image = SDL_CreateRGBSurface(SDL_SWSURFACE, 128, 64, 32,
                                    rmask, gmask, bmask, amask);
      color_Set(155,155,155, 255);
      switch(i)
      {
         case INITIAL_NEW: 
         {
            fnt.write(image, 0, 20, gettext("New"), true);
         }
         break;
         case INITIAL_OPTIONS:
         {
            fnt.write(image, 0, 20, gettext("Options"), true);
         }
         break;
         case INITIAL_EXIT:
         {
            fnt.write(image, 0, 20, gettext("Exit"), true);
         }
         break;
      }

      setTextureRGBA(image, textTexture[i]);
      SDL_FreeSurface(image);
   }
}

/********************************************************************
 *                               draw                               *
 ********************************************************************/
void initial::draw(cursor* cursors)
{
   int screenX = opt.getWindowX();
   int screenY = opt.getWindowY();
   int i;
   float y = (screenY / 2.0) - 72;
   float medX = (screenX / 2.0);
   float sum = 0;

   glClear (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

   draw2DMode();

   textureToScreen(bgTexture, 0, 0, screenX, screenY, 
                   BG_SIZE_X, BG_SIZE_Y);

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);

   for(i = 0; i <= INITIAL_EXIT; i++)
   {
      if(i == current)
      {
         sum = -16;
      }
      else
      {
         sum = 0;
      }
   
      glPushMatrix();
      
      textureToScreen(textTexture[i], 
                      (int)floor(medX-64-sum), 
                      (int)floor(y-32-(sum / 2.0)), 
                      (int)floor(medX+64+sum), 
                      (int)floor(y+32+(sum / 2.0)), 128, 64);

      
      glPopMatrix();
      y += 72;
   }

   /* Draw the Mouse Cursor */
   glPushMatrix();
      cursors->draw(mouseX, mouseY);
   glPopMatrix();

   glDisable(GL_BLEND);
   draw3DMode(FARVIEW);
   glFlush();
   SDL_GL_SwapBuffers();
}

/********************************************************************
 *                                 run                              *
 ********************************************************************/
int initial::run(cursor* cursors)
{
   int result = -1;
   current = -1;

   Uint8 mButton = 0;

   int time, varTime, sleepTime;

   int lastTime = 0;

   int screenX = opt.getWindowX();
   int screenY = opt.getWindowY();

   glDisable(GL_LIGHTING);
   glDisable(GL_FOG);

   while(result == -1)
   {
      time = SDL_GetTicks();
      varTime = time - lastTime;

      if( ((varTime)) >= UPDATE_RATE)
      {
         /* Update Clock */
         lastTime = time;
         draw(cursors);

         SDL_PumpEvents();
         mButton = SDL_GetMouseState(&mouseX,&mouseY);

         /* Verify Buttons */
         if( (mouseX >= ((screenX / 2.0) - 64)) &&
             (mouseX <= ((screenX / 2.0) + 64)) )
         {
            /* New Button */
            if( (mouseY >= ((screenY / 2.0) - 104)) &&
                (mouseY <= ((screenY / 2.0) - 40)) )
            {
               current = 0;
            }
            /* Options Button */
            else if( (mouseY >= ((screenY / 2.0) - 32)) &&
                     (mouseY <= ((screenY / 2.0) + 32)) )
            {
               current = 1;
            }
            /* Exit Button */
            else if( (mouseY >= ((screenY / 2.0) + 40)) &&
                (mouseY <= ((screenY / 2.0) + 104)) )
            {
               current = 2;
            }
            /* None */
            else
            {
               current = -1;
            }
         }
         else
         {
            current = -1;
         }

         //FIXME do button verify!
         if(mButton & SDL_BUTTON(1))
         {
            result = current;
         }
      }
      else
      {  
         /* Sleep until time pass */
         sleepTime = (int) ((UPDATE_RATE-1) - varTime);
         if( (sleepTime > 0) && (sleepTime < UPDATE_RATE))
         {
            SDL_Delay(sleepTime);
         }
      }
      
   }

   /* Wait Mouse Up */
   while(mButton & SDL_BUTTON(1))
   {
      SDL_Delay(20);
      SDL_PumpEvents();
      mButton = SDL_GetMouseState(&mouseX,&mouseY);
   }

   glEnable(GL_LIGHTING);
   glEnable(GL_FOG);

   return(result);
}


