
/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "maingui.h"

#include "../etc/draw.h"
#include "../etc/dirs.h"
#include "../etc/util.h"

#include "../lang/translate.h"

#include <iostream>
using namespace std;

/***********************************************************************
 *                             Constructor                             *
 ***********************************************************************/
mainGui::mainGui()
{
   pressing = false;

   /* Generate OpenGL Textures for Buttons */
   glGenTextures(MAIN_GUI_TOTAL_BUTTONS, textures);

   /* Load All Buttons */
   loadButtonTexture(MAIN_GUI_BUTTON_SHOOT, "gui/main/goal.png");
   loadButtonTexture(MAIN_GUI_BUTTON_REPLAY,"gui/main/replay.png");
}

/***********************************************************************
 *                              Destructor                             *
 ***********************************************************************/
mainGui::~mainGui()
{
   /* Free Button Textures */ 
   glDeleteTextures(MAIN_GUI_TOTAL_BUTTONS, textures);
}

/****************************************************************
 *                       loadButtonTexture                      *
 ****************************************************************/
void mainGui::loadButtonTexture(int button, string imageFile)
{
   dirs dir;
   SDL_Surface* img = IMG_Load(dir.getRealFile(imageFile).c_str());
   if(!img)
   {
      printf(gettext("Couldn't load texture: %s\n"), imageFile.c_str());
   }
   else
   {  
      setTextureRGBA(img, textures[button]);
      SDL_FreeSurface(img);
   }
}

/****************************************************************
 *                       getButtonPosition                      *
 ****************************************************************/
void mainGui::getButtonPosition(int button, float& posX, float& posY)
{
   switch(button)
   {
      case MAIN_GUI_BUTTON_SHOOT:
         posX = 33;
         posY = 60;
      break;
      case MAIN_GUI_BUTTON_REPLAY:
         posX = 33;
         posY = 130;
      break;
   }
}

/****************************************************************
 *                         verifyEvents                         *
 ****************************************************************/
int mainGui::verifyEvents(Uint8 mouseButton, int mouseX, int mouseY)
{
   int i;
   float posX=0, posY=0;

   /* Verify All Buttons */
   for(i = 0; i < MAIN_GUI_TOTAL_BUTTONS; i++)
   {
      /* Get Button Position */
      getButtonPosition(i, posX, posY);

      /* Verify if the mouse is pressing it */
      if(isPointIn(mouseX, mouseY, posX-32, posY-32, posX+32, posY+32))
      {
         if(mouseButton & SDL_BUTTON(1))
         {
            pressing = true;
         }
         else if( (pressing) && (!(mouseButton & SDL_BUTTON(1))) )
         {
            /* Done pressing, return the button constant value */
            pressing = false;
            return(i);
         }
      }      
   }

   if(!(mouseButton & SDL_BUTTON(1)))
   {
      pressing = false;
   }

   /* Nothing for now */
   return(MAIN_GUI_NOTHING);
}

/****************************************************************
 *                            render                            *
 ****************************************************************/
void mainGui::render(int mouseX, int mouseY)
{
   int i;
   float posX=0, posY=0;

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   glEnable(GL_TEXTURE_2D);

   /* Draw all buttons */
   for(i = 0; i < MAIN_GUI_TOTAL_BUTTONS; i++)
   {
      /* Get the center position */
      getButtonPosition(i, posX, posY);

      /* Draw the button */
      glBindTexture(GL_TEXTURE_2D, textures[i]);
      glPushMatrix();
         
         /* Put at position */
         glTranslatef(posX, opt.getWindowY() - posY, 0.0);

         /* Verify if mouse in */
         if(isPointIn(mouseX, mouseY, posX-32, posY-32, posX+32, posY+32) )
         {
            glScalef(0.8, 0.8, 1.0);
         }

         /* Render the quad */
         glBegin(GL_QUADS);
            glTexCoord2d(0,1);
            glVertex2f(-32,-32);
            glTexCoord2d(0,0);
            glVertex2f(-32,32);
            glTexCoord2d(1,0);
            glVertex2f(32,32);
            glTexCoord2d(1,1);
            glVertex2f(32,-32);
         glEnd();
      glPopMatrix();
   }
}

