#ifndef _xut_main_gui_h
#define _xut_main_gui_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>

#include <iostream>
#include <string>
using namespace std;

#include "../etc/options.h"

#define MAIN_GUI_BUTTON_SHOOT    0
#define MAIN_GUI_BUTTON_REPLAY   1 
#define MAIN_GUI_TOTAL_BUTTONS   2

#define MAIN_GUI_NOTHING       200

/*! The mainGui have all main interactive GUI
 * needed by xut */
class mainGui
{
   public:
      /*! Costructor */
      mainGui();
      /*! Destructor */
      ~mainGui();

      /*! Verify Events at the main GUI
       * \param mouseButton -> current mouse button state
       * \param mouseX -> current mouseX position 
       * \param mouseY -> current mouseY position */
      int verifyEvents(Uint8 mouseButton, int mouseX, int mouseY);

      /*! Render the main GUI to the screen
       * \param mouseX -> current mouseX position
       * \param mouseY -> current mouseY position */
      void render(int mouseX, int mouseY);

   protected:

      options opt;

      /*! Load the button Texture
       * \param button -> internal button ID
       * \param imageFile -> fileName of the image to load */
      void loadButtonTexture(int button, string imageFile);

      /*! Get the button center position
       * \param button -> internal button ID 
       * \param posX -> receives X center position of the button
       * \param posY -> receives Y center position of the button */
      void getButtonPosition(int button, float& posX, float& posY);

      GLuint textures[MAIN_GUI_TOTAL_BUTTONS];  /**< The replay texture */
      bool pressing;               /**< True if is pressing a button */

};


#endif

