
/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "optscreen.h"
#include "../etc/draw.h"
#include "../lang/translate.h"
#include "dntfont.h"

#include <iostream>
using namespace std;

#define BG_SIZE_X 800
#define BG_SIZE_Y 600

#define OPT_SCREEN_STATE_INITIAL   0
#define OPT_SCREEN_STATE_GRAPHICS  1
#define OPT_SCREEN_STATE_SOUND     2
#define OPT_SCREEN_STATE_GAME      3
#define OPT_SCREEN_STATE_KEYS      4
#define OPT_SCREEN_STATE_LANGUAGE  5
#define OPT_SCREEN_STATE_GET_KEY   6

/***********************************************************************
 *                             Constructor                             *
 ***********************************************************************/
optScreen::optScreen(GLdouble* proj,GLdouble* modl, GLint* vPort, GLuint bg)
{
   state = OPT_SCREEN_STATE_INITIAL;

   current = -1;

   projection = proj;
   modelView = modl;
   viewPort = vPort;
   bgTexture = bg;

   opt.defineResolutions();

   loadTextTextures();
}

/********************************************************************
 *                              Destructor                          *
 ********************************************************************/
optScreen::~optScreen()
{
   glDeleteTextures(OPT_SCREEN_TOTAL_TEXTURES, textTexture);
}

/********************************************************************
 *                         loadTextTexture                          *
 ********************************************************************/
void optScreen::loadTextTexture(int index)
{
   /* Define Machine Bit Order */
   Uint32 rmask, gmask, bmask, amask;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
   rmask = 0xff000000;
   gmask = 0x00ff0000;
   bmask = 0x0000ff00;
   amask = 0x000000ff;
#else
   rmask = 0x000000ff;
   gmask = 0x0000ff00;
   bmask = 0x00ff0000;
   amask = 0xff000000;
#endif

   /* Set Font */
   dntFont fnt;
   
   if(index <= OPT_SCREEN_BACK)
   {
      fnt.defineFont(DNT_FONT_ARIAL, 26);      
   }
   else
   {
      fnt.defineFont(DNT_FONT_ARIAL, 14);      
   }
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   SDL_Surface* image;
   
   image = SDL_CreateRGBSurface(SDL_SWSURFACE, 256, 64, 32,
         rmask, gmask, bmask, amask);

   if(index >= OPT_SCREEN_CAMERA_UP)
   {
      color_Set(185,20,20, 255);
      fnt.defineFontStyle(DNT_FONT_STYLE_BOLD);
   }
   else
   {
      color_Set(255,155,2, 255);
      fnt.defineFontStyle(DNT_FONT_STYLE_NORMAL);
   }

   fnt.write(image, 0, 20, getTextValue(index), true);

   setTextureRGBA(image, textTexture[index]);
   SDL_FreeSurface(image);
}

/********************************************************************
 *                           getTextValue                           *
 ********************************************************************/
string optScreen::getTextValue(int index)
{
   char text[255];
   int wX=0, wY=0;

   switch(index)
   {
      case OPT_SCREEN_GRAPHICS:
         return(gettext("Graphics"));
      break;
      case OPT_SCREEN_SOUND: 
         return(gettext("Sound"));
      break;
      case OPT_SCREEN_GAME: 
         return(gettext("Game"));
      break;
      case OPT_SCREEN_KEYS: 
         return(gettext("Keys"));
      break;
      case OPT_SCREEN_LANGUAGE:
         return(gettext("Language"));
      break;
      case OPT_SCREEN_BACK:
         return(gettext("Back"));
      break;
      case OPT_SCREEN_PRESS_KEY:
         return(gettext("Waiting for key..."));
      break;

      /* Graphics options texts */
      case OPT_SCREEN_ANTI_ALIASING:
         if(opt.getAntiAliasing() > 0)
         {
            sprintf(text, "%s:|%dx", gettext("AntiAliasing"), 
                    opt.getAntiAliasing());
         }
         else
         {
            sprintf(text, "%s:|%s", gettext("AntiAliasing"), 
                    gettext("Disabled"));
         }
         return(text);
      break;
      case OPT_SCREEN_RESOLUTION:
         opt.getPotentialWindowSize(wX, wY);
         sprintf(text, "%s:|%dx%d", gettext("Resolution"), wX, wY);
         return(text);
      break;
      case OPT_SCREEN_FULLSCREEN:
         sprintf(text, "%s:|%s", gettext("FullScreen"), 
                 opt.getFullScreen()?gettext("Enabled"):gettext("Disabled"));
         return(text);
      break;
      case OPT_SCREEN_FIELD_DETAIL:
         switch(opt.getFieldDetail())
         {
            case XUT_OPTIONS_FIELD_DETAIL_LOW:
               sprintf(text, "%s:|%s", gettext("Field Detail"), 
                       gettext("Low"));
            break;
            case XUT_OPTIONS_FIELD_DETAIL_NORMAL:
               sprintf(text, "%s:|%s", gettext("Field Detail"), 
                       gettext("Normal"));
            break;
            case XUT_OPTIONS_FIELD_DETAIL_HIGH:
               sprintf(text, "%s:|%s", gettext("Field Detail"), 
                       gettext("High"));
            break;
         }
         return(text);
      break;
      case OPT_SCREEN_CAMERA_TYPE:
         switch(opt.getCameraType())
         {
            case XUT_OPTIONS_CAMERA_TYPE_STATIC:
               sprintf(text, "%s:|%s", gettext("Camera Type"), 
                       gettext("Static"));
            break;
            case XUT_OPTIONS_CAMERA_TYPE_FOLLOW_FOULS:
               sprintf(text, "%s:|%s", gettext("Camera Type"), 
                       gettext("Normal"));
            break;
            case XUT_OPTIONS_CAMERA_TYPE_FOLLOW_BALL:
               sprintf(text, "%s:|%s", gettext("Camera Type"), 
                       gettext("Follow Ball"));
            break;
         }
         return(text);
      break;
   
      /* Sound Options texts */
      case OPT_SCREEN_SOUND_VOLUME: 
         sprintf(text, "%s:|%d", gettext("Sound Volume"), opt.getSoundVolume());
         return(text);
      break;

      /* Sound Options texts */
      case OPT_SCREEN_LANGUAGE_NAME: 
         return(opt.getLanguageName());
      break;

      /* Key Options texts  */
      case OPT_SCREEN_CAMERA_UP: 
         sprintf(text, "%s:|%s", gettext("Camera Up"), 
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_UP)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_DOWN: 
         sprintf(text, "%s:|%s", gettext("Camera Down"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_DOWN)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_LEFT: 
         sprintf(text, "%s:|%s", gettext("Camera Left"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_LEFT)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_RIGHT:
         sprintf(text, "%s:|%s", gettext("Camera Right"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_RIGHT)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_ROTATE_LEFT:
         sprintf(text, "%s:|%s", gettext("Camera Rotate Left"),
                SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_ROTATE_LEFT)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_ROTATE_RIGHT:
         sprintf(text, "%s:|%s", gettext("Camera Rotate Right"),
               SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_ROTATE_RIGHT)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_ROTATE_UP: 
         sprintf(text, "%s:|%s", gettext("Camera Rotate Up"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_ROTATE_UP)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_ROTATE_DOWN: 
         sprintf(text, "%s:|%s", gettext("Camera Rotate Down"),
                SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_ROTATE_DOWN)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_ZOOM_UP:
         sprintf(text, "%s:|%s", gettext("Camera Zoom Up"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_ZOOM_UP)));
         return(text);
      break;
      case OPT_SCREEN_CAMERA_ZOOM_DOWN:
         sprintf(text, "%s:|%s", gettext("Camera Zoom Down"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_CAMERA_ZOOM_DOWN)));
         return(text);
      break;
      case OPT_SCREEN_WILL_SHOOT:
         sprintf(text, "%s:|%s", gettext("Will Shoot"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_WILL_SHOOT)));
         return(text);
      break;
      case OPT_SCREEN_REPLAY: 
         sprintf(text, "%s:|%s", gettext("Replay"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_REPLAY)));
         return(text);
      break;
      case OPT_SCREEN_ROTATE_LEFT: 
         sprintf(text, "%s:|%s", gettext("Rotate Left"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_ROTATE_LEFT)));
         return(text);
      break;
      case OPT_SCREEN_ROTATE_RIGHT:
         sprintf(text, "%s:|%s", gettext("Rotate Right"),
                 SDL_GetKeyName((SDLKey)opt.getKey(XUT_KEY_ROTATE_RIGHT)));
         return(text);
      break;
   }

   return("");
}

/********************************************************************
 *                        loadTextTextures                          *
 ********************************************************************/
void optScreen::loadTextTextures()
{
   int i;

   /* Generate All Textures */
   glGenTextures(OPT_SCREEN_TOTAL_TEXTURES, textTexture);

   for(i = 0; i < OPT_SCREEN_TOTAL_TEXTURES; i++)
   {
      loadTextTexture(i);
   }
}

/********************************************************************
 *                       reloadTextTextures                         *
 ********************************************************************/
void optScreen::reloadTextTextures()
{
   int i;

   for(i = 0; i < OPT_SCREEN_TOTAL_TEXTURES; i++)
   {
      loadTextTexture(i);
   }
}

/********************************************************************
 *                         renderOptions                            *
 ********************************************************************/
void optScreen::renderOptions(int initial, int end)
{
   int screenX = opt.getWindowX();
   int screenY = opt.getWindowY();
   int i;
   float y;
   float medX = (screenX / 2.0);
   float sum = 0;

   y = (initial < OPT_SCREEN_CAMERA_UP) ? ((screenY / 2.0) - 72) : 20;
   
   for(i=0; i<=(end - initial);i++)
   {
      sum = 0;
      if(i+initial == current)
      {
         sum = (i+initial >= OPT_SCREEN_BACK)?-8:-16;
         if(initial > OPT_SCREEN_BACK)
         {
            glColor3f(0.0f, 1.0f, 1.0f);
         }
      }
      else
      {
         glColor3f(1.0f, 1.0f, 1.0f);
      }
      glPushMatrix();
         textureToScreen(textTexture[i+initial], 
               (int)floor(medX-128-sum), 
               (int)floor(y-32-(sum / 2.0)), 
               (int)floor(medX+128+sum), 
               (int)floor(y+32+(sum / 2.0)), 256, 64);
      glPopMatrix();
      
      y += (initial >= OPT_SCREEN_CAMERA_UP)?42:72;
   }

   glColor3f(1.0f, 1.0f, 1.0f);
}

/********************************************************************
 *                               draw                               *
 ********************************************************************/
void optScreen::draw(cursor* cursors)
{
   int screenX = opt.getWindowX();
   int screenY = opt.getWindowY();
   int sum = 0;

   glClear (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

   draw2DMode();

   textureToScreen(bgTexture, 0, 0, screenX, screenY, 
                   BG_SIZE_X, BG_SIZE_Y);

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);

   /* Render Back Button */
   sum = 0;
   glPushMatrix();
      if(current == OPT_SCREEN_BACK)
      {
         sum = -16;
      }
      textureToScreen(textTexture[OPT_SCREEN_BACK], 
                      sum-64, opt.getWindowY()-64-sum/2,
                      172+sum, opt.getWindowY()+sum/2, 256,64);
   glPopMatrix();

   /* Render All Other Buttons, related to the current state */
   switch(state)
   {
      case OPT_SCREEN_STATE_INITIAL:
      {
         renderOptions(0, OPT_SCREEN_LANGUAGE);
      }
      break;
      case OPT_SCREEN_STATE_GRAPHICS:
      {
         renderOptions(OPT_SCREEN_ANTI_ALIASING, OPT_SCREEN_CAMERA_TYPE);
      }
      break;
      case OPT_SCREEN_STATE_SOUND:
      {
         renderOptions(OPT_SCREEN_SOUND_VOLUME, OPT_SCREEN_SOUND_VOLUME);
      }
      break;
      case OPT_SCREEN_STATE_GAME:
      {
         //TODO nothing for now.
      }
      break;
      case OPT_SCREEN_STATE_LANGUAGE:
      {
         renderOptions(OPT_SCREEN_LANGUAGE_NAME, OPT_SCREEN_LANGUAGE_NAME);
      }
      break;
      case OPT_SCREEN_STATE_KEYS:
      {
         renderOptions(OPT_SCREEN_CAMERA_UP, OPT_SCREEN_ROTATE_RIGHT);
      }
      break;
      case OPT_SCREEN_STATE_GET_KEY:
      {
         glPushMatrix();
         textureToScreen(textTexture[OPT_SCREEN_PRESS_KEY], 
                         screenX/2-128, screenY/2-32,
                         screenX/2+128, screenY/2+32, 256,64);
         glPopMatrix();
      }
      break;
   }

   /* Draw the Mouse Cursor */
   glPushMatrix();
      cursors->draw(mouseX, mouseY);
   glPopMatrix();

   glDisable(GL_BLEND);
   draw3DMode(FARVIEW);
   glFlush();
   SDL_GL_SwapBuffers();
}

/********************************************************************
 *                                 run                              *
 ********************************************************************/
bool optScreen::verifyInput()
{
   int screenY = opt.getWindowY();
   int prevState = state;

   current = -1;

   /* Verify Back Button */
   if( (mouseX >= 0) && (mouseX <= 128) && 
       (mouseY >= (screenY - 64) ) && (mouseY <= screenY) )
   {
      while(mButton & SDL_BUTTON(1))
      {
         SDL_Delay(20);
         SDL_PumpEvents();
         mButton = SDL_GetMouseState(&mouseX,&mouseY);            
         if(prevState == OPT_SCREEN_STATE_INITIAL)
         {
            /* back at initial, so done! */
            return(true);
         }
         /* back at any other state, so to previous state */
         if(prevState != OPT_SCREEN_STATE_GET_KEY)
         {
            state = OPT_SCREEN_STATE_INITIAL;
         }
         else
         {
            state = OPT_SCREEN_STATE_KEYS;
         }
      }
      current = OPT_SCREEN_BACK;
      return(false);
   }
   
   /* Verify other buttons */
   switch(state)
   {
      case OPT_SCREEN_STATE_INITIAL:
      {
         verifyMouseInput(0, OPT_SCREEN_LANGUAGE);
      }
      break;
      case OPT_SCREEN_STATE_GRAPHICS:
      {
         verifyMouseInput(OPT_SCREEN_ANTI_ALIASING, OPT_SCREEN_CAMERA_TYPE);
      }
      break;
      case OPT_SCREEN_STATE_SOUND:
      {
         verifyMouseInput(OPT_SCREEN_SOUND_VOLUME, OPT_SCREEN_SOUND_VOLUME);
      }
      break;
      case OPT_SCREEN_STATE_GAME:
      {
         //TODO nothing for now.
      }
      break;
      case OPT_SCREEN_STATE_LANGUAGE:
      {
         verifyMouseInput(OPT_SCREEN_LANGUAGE_NAME, OPT_SCREEN_LANGUAGE_NAME);
      }
      break;
      case OPT_SCREEN_STATE_KEYS:
      {
         verifyMouseInput(OPT_SCREEN_CAMERA_UP, OPT_SCREEN_ROTATE_RIGHT);
      }
      break;
      case OPT_SCREEN_STATE_GET_KEY:
      {
         SDL_Event event;
         while(SDL_PollEvent(&event))
         {
            if(event.type == SDL_KEYUP)
            {
               opt.setKey(currentKeyCheck, event.key.keysym.sym);
               state = OPT_SCREEN_STATE_KEYS;
               SDL_EventState(SDL_KEYUP, SDL_IGNORE);
               loadTextTexture(currentKeyCheck+OPT_SCREEN_CAMERA_UP);
            }
         }
      }
      break;
   }

   return(false);
}

/********************************************************************
 *                           verifyMouseInput                       *
 ********************************************************************/
void optScreen::verifyMouseInput(int init, int end)
{
   int screenX = opt.getWindowX();
   int screenY = opt.getWindowY();
   int i;

   int initY = (init < OPT_SCREEN_CAMERA_UP) ? ((screenY / 2) - 82) : 10;
   int sumY = (init >= OPT_SCREEN_CAMERA_UP)?42:72;

   for(i = 0; i <= (end-init); i++)
   {
      if( (mouseX >= ((screenX / 2.0) - 64)) &&
          (mouseX <= ((screenX / 2.0) + 64)) &&
          (mouseY <= (initY + (i+1)*sumY)) &&
          (mouseY >= (initY + i*sumY)) )
      {
         current = i+init;
         
         if(mButton & SDL_BUTTON(1))
         {
            takeAction();
            /* Wait for the mouse release */
            while(mButton & SDL_BUTTON(1))
            {
               SDL_Delay(20);
               SDL_PumpEvents();
               mButton = SDL_GetMouseState(&mouseX,&mouseY);            
            }
         }
      }
   }
}

/********************************************************************
 *                             takeAction                           *
 ********************************************************************/
void optScreen::takeAction()
{
   if(current <= OPT_SCREEN_LANGUAGE)
   {
      state = current+1;
   }
   else if(current == OPT_SCREEN_LANGUAGE_NAME)
   {
      /* Set the new language */
      opt.setLanguage(opt.getLanguage()+1);
      /* And reload text textures with the new language */
      reloadTextTextures();
   }
   else if( (current >= OPT_SCREEN_ANTI_ALIASING) && 
            (current <= OPT_SCREEN_CAMERA_TYPE) )
   {
      switch(current)
      {
         case OPT_SCREEN_ANTI_ALIASING:
            opt.setAntiAliasing();
         break;
         case OPT_SCREEN_RESOLUTION:
            opt.setWindowSize();
         break;
         case OPT_SCREEN_FULLSCREEN:
            opt.setFullScreen();
         break;
         case OPT_SCREEN_FIELD_DETAIL:
            opt.setFieldDetail();
         break;
         case OPT_SCREEN_CAMERA_TYPE:
            opt.setCameraType();
         break;
      }
      loadTextTexture(current);
   }
   else if(current == OPT_SCREEN_SOUND_VOLUME)
   {
      opt.setSoundVolume();
      loadTextTexture(current);
   }
   else if( (current >= OPT_SCREEN_CAMERA_UP) &&
            (current <= OPT_SCREEN_ROTATE_RIGHT) )
   {
      state = OPT_SCREEN_STATE_GET_KEY;
      currentKeyCheck = current - OPT_SCREEN_CAMERA_UP;
      /* Enable keyUp events to get the keys */
      SDL_EventState(SDL_KEYUP, SDL_ENABLE);
   }
}

/********************************************************************
 *                                 run                              *
 ********************************************************************/
void optScreen::run(cursor* cursors)
{
   bool done=false;
   current = -1;

   mButton = 0;

   int time, varTime, sleepTime;

   int lastTime = 0;

   glDisable(GL_LIGHTING);
   glDisable(GL_FOG);

   while(!done)
   {
      time = SDL_GetTicks();
      varTime = time - lastTime;

      if( ((varTime)) >= UPDATE_RATE)
      {
         /* Update Clock */
         lastTime = time;
         draw(cursors);

         SDL_PumpEvents();
         mButton = SDL_GetMouseState(&mouseX,&mouseY);

         /* Verify Buttons */
         done = verifyInput();
      }
      else
      {  
         /* Sleep until time pass */
         sleepTime = (int) ((UPDATE_RATE-1) - varTime);
         if( (sleepTime > 0) && (sleepTime < UPDATE_RATE))
         {
            SDL_Delay(sleepTime);
         }
      }
      
   }

   /* Wait Mouse Up */
   while(mButton & SDL_BUTTON(1))
   {
      SDL_Delay(20);
      SDL_PumpEvents();
      mButton = SDL_GetMouseState(&mouseX,&mouseY);
   }

   glEnable(GL_LIGHTING);
   glEnable(GL_FOG);

   /* Save the current options */
   opt.save();
}


