#ifndef _selector_h
#define _selector_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL_opengl.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <string>
using namespace std;

#include "../etc/cursor.h"
#include "../etc/options.h"

#define SELECTOR_CANCELED  0     /**< When Canceled the Selector */
#define SELECTOR_DONE      1     /**< When Done with the Selector */

#define SELECTOR_TOTAL_BUTTONS 4 /**< Total Buttons At the Selector */

/*! The teamSel is the team information class for the selector */
class teamSel
{
   public:
      string fileName;  /**< FileName of the Team */
      string name;      /**< Name of the Team */
      GLuint logo;      /**< Logo Texture of the Team */
};

/*! The selector class is used for the user select both teams to play. */
class selector
{
   public:
      /*! Constructor */
      selector(GLuint bg);
      /*! Destructor */
      ~selector();

      /*! Run the Team Selector
       * \param cursors -> mouse cursors
       * \return -> selector constant 
       *           (SELECTOR_DONE or SELECTOR_CANCELED) */
      int run(cursor* cursors);

      /*! Get the fileName of the selected teamA (if one)
       * \return fileName of the selected teamA */
      string getTeamA();

      /*! Get the fileName of the selected teamB (if one)
       * \return fileName of the selected teamB */
      string getTeamB(); 

   protected:

      /*! Draw the selector 
       * \param cursors -> current mouse cursors */
      void draw(cursor* cursors);

      /*! Verify Input at the selector */
      void verifyIO();

      /*! Load the button Texture
       * \param button -> internal button ID
       * \param imageFile -> fileName of the image to load */
      void loadButtonTexture(int button, string imageFile);

      /*! Get the button center position
       * \param button -> internal button ID 
       * \param posX -> receives X center position of the button
       * \param posY -> receives Y center position of the button */
      void getButtonPosition(int button, float& posX, float& posY);

      /*! reload the Team Name Texture */
      void reloadNameTexture();

      /*! reload the Team Message Texture */
      void reloadMessageTexture();

      /*! Draw the team logo to the screen */
      void drawLogo();

      /*! Init the Animation of team selections change */
      void beginAnimation();

      string teamA;     /**< Current Selected TeamA fileName */
      string teamB;     /**< Current Selected TeamB fileName */

      teamSel* teams;   /**< Possible teams to select */
      int totalTeams;   /**< Total avaible teams */
      int curTeam;      /**< Current Team Displayed */
      int prevTeam;     /**< Previous Team Displayed */

      bool firstTeam;   /**< If getting first team */
      
      int mouseX,             /**< Current MouseX */
          mouseY;             /**< Current MouseY */

      int result;       /**< The result of the run */

      GLuint buttonTextures[SELECTOR_TOTAL_BUTTONS]; /**< Button Textures */
      GLuint nameTexture; /**< Name Texture */
      GLuint messageTexture; /**< The Message Texture */
      GLuint bgTexture;  /**< BackGround Texture */

      int currentButton;  /**< Current Button Mouse is At */
      int selectedButton; /**< Current Button Mouse is Pressing */

      SDL_Surface* nameImage; /**< Image bellow the name  */

      bool done;        /**< Done Selecting Team */

      int state;        /**< Selector Internal State */

      float logoCurScale;  /**< Current Logo Scale Factor */
      float logoCurAngle;  /**< Current Logo Angle */
      bool curSignal;      /**< Current signal used */

      options opt;
};


#endif

