#ifndef _videocontroller_h
#define _videocontroller_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>
#include <string>
using namespace std;

#include "../etc/draw.h"
#include "../etc/options.h"

#define VIDEO_CONTROLLER_TOTAL_BUTTONS   9

#define VIDEO_CONTROLLER_NOTHING        -1
#define VIDEO_CONTROLLER_STOP            0
#define VIDEO_CONTROLLER_PLAY            1
#define VIDEO_CONTROLLER_PAUSE           2
#define VIDEO_CONTROLLER_FORWARD         3
#define VIDEO_CONTROLLER_BACKWARD        4
#define VIDEO_CONTROLLER_HALF_SPEED      5
#define VIDEO_CONTROLLER_NORMAL_SPEED    6
#define VIDEO_CONTROLLER_DOUBLE_SPEED    7
#define VIDEO_CONTROLLER_SHUTDOWN        8

/*! The video Controller class implements a video controller,
 * with stop, forward, backward, play, pause buttons, shutdown
 * and speed buttons */
class videoController
{
   public:
      /*! Constructor */
      videoController();
      
      /*! Destructor */
      ~videoController();

      /*! Verify Button press at the controller
       * \param mButton -> mouse button state
       * \param mouseX -> mouse X position
       * \param mouseY -> mouse Y position
       * \return -> videoController constant */
      int verifyButtons(Uint8 mButton, int mouseX, int mouseY);

      /*! Render the controller to the screen */
      void render();

   protected:

      /*! Get the button center position
       * \param button -> internal button ID 
       * \param posX -> receives X center position of the button
       * \param posY -> receives Y center position of the button */
      void getButtonPosition(int button, float& posX, float& posY);

      /*! Load the button Texture
       * \param button -> internal button ID
       * \param imageFile -> fileName of the image to load */
      void loadButtonTexture(int button, string imageFile);

      GLuint textures[VIDEO_CONTROLLER_TOTAL_BUTTONS]; /**< button textures */
      int currentButton;      /**< Current selected button */
      options opt;
};


#endif

