#include "forceio.h"
#include <math.h>

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#define VALUE_NOT_DEFINED -1
#include <SDL/SDL_opengl.h>

/*************************************************************
 *                       Constructor                         *
 *************************************************************/
forceIO::forceIO()
{
   clear();
}

/*************************************************************
 *                        Destructor                         *
 *************************************************************/
forceIO::~forceIO()
{
}

/*************************************************************
 *                          clear                            *
 *************************************************************/
void forceIO::clear()
{
   initialX = VALUE_NOT_DEFINED;
   initialZ = VALUE_NOT_DEFINED;

   finalX = VALUE_NOT_DEFINED;
   finalZ = VALUE_NOT_DEFINED;
}

/*************************************************************
 *                       setInitial                          *
 *************************************************************/
void forceIO::setInitial(float x, float z)
{
   initialX = x;
   initialZ = z;
}

/*************************************************************
 *                        setFinal                           *
 *************************************************************/
void forceIO::setFinal(float x, float z)
{
   finalX = x;
   finalZ = z;
}

/*************************************************************
 *                         defined                           *
 *************************************************************/
bool forceIO::defined()
{
   return( (initialX != VALUE_NOT_DEFINED) &&
           (initialZ != VALUE_NOT_DEFINED) &&
           (finalX != VALUE_NOT_DEFINED) &&
           (finalZ != VALUE_NOT_DEFINED) );
}

/*************************************************************
 *                        getForce                           *
 *************************************************************/
void forceIO::getForce(float& forceValue, float& directionX, 
                       float& directionZ)
{
   if(!defined())
   {
      /* Some value isn't defined yet! */
      forceValue = 0;
      directionX = 0;
      directionZ = 0;
      return;
   }

   float dX = (finalX - initialX) / 6.0;
   float dZ = (finalZ - initialZ) / 6.0;
   forceValue = sqrt( (dX*dX) + (dZ*dZ) );
   if(forceValue != 0)
   {
      directionX = -dX / forceValue;
      directionZ = -dZ / forceValue;
   }
   else
   {
      directionX = 0;
      directionZ = 0;
   }

}

/*************************************************************
 *                         render                            *
 *************************************************************/
void forceIO::render()
{
   if(defined())
   {
      glBegin(GL_LINES);
         glVertex3f(initialX, 0.5, initialZ);
         glVertex3f(finalX, 0.5, finalZ);
      glEnd();
   }
}

